@@ -1,4 +1,4 @@
-from ctypes import POINTER, c_void_p
+from ctypes import *
 
 def test_pointer_subclasses():
     Void_pp = POINTER(c_void_p)
