@@ -10,6 +10,9 @@ import weakref
 import array
 from test import test_support
 import io
+import copy
+import pickle
+import warnings
 
 
 class AbstractMemoryTests:
@@ -361,6 +364,25 @@ class BytesMemorySliceSliceTest(unittest.TestCase,
     #pass
 
 
+class OtherTest(unittest.TestCase):
+    def test_copy(self):
+        m = memoryview(b'abc')
+        with self.assertRaises(TypeError), warnings.catch_warnings():
+            warnings.filterwarnings('ignore', ".*memoryview", DeprecationWarning)
+            copy.copy(m)
+
+    @test_support.cpython_only
+    def test_pickle(self):
+        m = memoryview(b'abc')
+        for proto in range(2):
+            with self.assertRaises(TypeError):
+                pickle.dumps(m, proto)
+        with test_support.check_py3k_warnings(
+                (".*memoryview", DeprecationWarning)):
+            pickle.dumps(m, 2)
+
+
+
 def test_main():
     test_support.run_unittest(__name__)
 
