@@ -115,6 +115,7 @@ testmap = [
     RegrTest('test_augassign.py', core=True),
     RegrTest('test_base64.py', usemodules='struct'),
     RegrTest('test_bastion.py'),
+    RegrTest('test_bdb.py', usemodules='bdb'),
     RegrTest('test_bigaddrspace.py'),
     RegrTest('test_bigmem.py'),
     RegrTest('test_binascii.py', usemodules='binascii'),
@@ -271,6 +272,7 @@ testmap = [
     RegrTest('test_import.py', core=True),
     RegrTest('test_importhooks.py', core=True),
     RegrTest('test_importlib.py'),
+    RegrTest('test_import_magic.py'),
     RegrTest('test_index.py'),
     RegrTest('test_inspect.py'),
     RegrTest('test_int.py', core=True),
@@ -369,6 +371,7 @@ testmap = [
     RegrTest('test_random.py'),
     RegrTest('test_re.py', core=True),
     RegrTest('test_readline.py'),
+    RegrTest('test_regrtest.py'),
     RegrTest('test_repr.py', core=True),
     RegrTest('test_resource.py'),
     RegrTest('test_rfc822.py'),
@@ -429,6 +432,7 @@ testmap = [
     RegrTest('test_tcl.py'),
     RegrTest('test_telnetlib.py'),
     RegrTest('test_tempfile.py'),
+    RegrTest('test_test_support.py'),
     RegrTest('test_textwrap.py'),
     RegrTest('test_thread.py', usemodules="thread", core=True),
     RegrTest('test_threaded_import.py', usemodules="thread", core=True),
