@@ -1,5 +1,5 @@
 from __future__ import print_function
-import sys, shutil, os, tempfile, hashlib
+import sys, shutil, os, tempfile, hashlib, collections
 import sysconfig
 from os.path import join
 
@@ -22,18 +22,22 @@ class MissingDependenciesError(Exception):
     pass
 
 
-cffi_build_scripts = {
-    "_ssl": "_ssl_build.py",
-    "sqlite3": "_sqlite3_build.py",
-    "audioop": "_audioop_build.py",
-    "_tkinter": "_tkinter/tklib_build.py",
-    "curses": "_curses_build.py" if sys.platform != "win32" else None,
-    "syslog": "_syslog_build.py" if sys.platform != "win32" else None,
-    "gdbm": "_gdbm_build.py"  if sys.platform != "win32" else None,
-    "grp": "_pwdgrp_build.py" if sys.platform != "win32" else None,
-    "resource": "_resource_build.py" if sys.platform != "win32" else None,
-    "xx": None,    # for testing: 'None' should be completely ignored
-    }
+cffi_build_scripts = collections.OrderedDict({
+    ("_ctypes._ctypes_cffi",
+     "_ctypes/_ctypes_build.py" if sys.platform == 'darwin' else None),
+    ("_pypy_util_cffi_inner", "_pypy_util_build.py"), # this needs to come before ssl
+    ("_ssl", "_ssl_build.py"),
+    ("sqlite3", "_sqlite3_build.py"),
+    ("audioop", "_audioop_build.py"),
+    ("_tkinter", "_tkinter/tklib_build.py"),
+    ("curses", "_curses_build.py" if sys.platform != "win32" else None),
+    ("syslog", "_syslog_build.py" if sys.platform != "win32" else None),
+    ("gdbm", "_gdbm_build.py"  if sys.platform != "win32" else None),
+    ("grp", "_pwdgrp_build.py" if sys.platform != "win32" else None),
+    ("resource", "_resource_build.py" if sys.platform != "win32" else None),
+    # ("_decimal", "_decimal_build.py"),  # issue 3024
+    ("xx", None),    # for testing: 'None' should be completely ignored
+    })
 
 # for distribution, we may want to fetch dependencies not provided by
 # the OS, such as a recent openssl/libressl.
@@ -55,21 +59,32 @@ cffi_dependencies = {
               ['make', '-s', '-j', str(multiprocessing.cpu_count())],
               ['make', 'install', 'DESTDIR={}/'.format(deps_destdir)],
              ]),
-    '_ssl': ('http://distfiles.macports.org/openssl/openssl-1.1.1f.tar.gz',
-             '186c6bfe6ecfba7a5b48c47f8a1673d0f3b0e5ba2e25602dd23b629975da3f35',
-             [['./config', '--prefix=/usr', 'no-shared'],
+    '_ssl1': ('http://artfiles.org/openssl.org/source/openssl-1.1.1n.tar.gz',
+             '40dceb51a4f6a5275bde0e6bf20ef4b91bfc32ed57c0552e2e8e15463372b17a',
+             [
+              ['./config', '--prefix=/usr', 'no-shared'],
               ['make', '-s', '-j', str(multiprocessing.cpu_count())],
               ['make', 'install', 'DESTDIR={}/'.format(deps_destdir)],
              ]),
+    '_ssl3': ('http://artfiles.org/openssl.org/source/openssl-3.0.1.tar.gz',
+              'c311ad853353bce796edad01a862c50a8a587f62e7e2100ef465ab53ec9b06d1',
+              [
+               ['./config', '--prefix=/usr', 'no-shared', 'enable-fips'],
+               ['make', '-s', '-j', str(multiprocessing.cpu_count())],
+               ['make', 'install', 'DESTDIR={}/'.format(deps_destdir)],
+              ]),
+}
+cffi_dependencies['_ssl'] = cffi_dependencies['_ssl1']
+
+if sys.platform == 'darwin':
     # this does not compile on the buildbot, linker is missing '_history_list'
-    'gdbm': ('http://distfiles.macports.org/gdbm/gdbm-1.18.1.tar.gz',
+    cffi_dependencies['gdbm'] = (
+              'http://distfiles.macports.org/gdbm/gdbm-1.18.1.tar.gz',
               '86e613527e5dba544e73208f42b78b7c022d4fa5a6d5498bf18c8d6f745b91dc',
               [configure_args + ['--without-readline'],
               ['make', '-s', '-j', str(multiprocessing.cpu_count())],
               ['make', 'install', 'DESTDIR={}/'.format(deps_destdir)],
-             ]),
-}
-
+             ])
 
 def _unpack_tarfile(filename, extract_dir):
     """Unpack tar/tar.gz/tar.bz2/tar.xz `filename` to `extract_dir`
@@ -165,6 +180,7 @@ def _build_dependency(name, patches=[]):
 def create_cffi_import_libraries(pypy_c, options, basedir, only=None,
                                  embed_dependencies=False, rebuild=False):
     from rpython.tool.runsubprocess import run_subprocess
+    print('calling create_cffi_import_libraries with "embed_dependencies"', embed_dependencies)
 
     shutil.rmtree(str(join(basedir,'lib_pypy','__pycache__')),
                   ignore_errors=True)
@@ -214,19 +230,19 @@ def create_cffi_import_libraries(pypy_c, options, basedir, only=None,
                 print(stderr.decode('utf-8'))
                 continue
 
-            env['CPPFLAGS'] = \
-                '-I{}/usr/include {}'.format(deps_destdir, env.get('CPPFLAGS', ''))
-            env['LDFLAGS'] = \
-                '-L{}/usr/lib {}'.format(deps_destdir, env.get('LDFLAGS', ''))
+            env['CPPFLAGS'] = '-I{}/usr/include {}'.format(
+                            deps_destdir, env.get('CPPFLAGS', ''))
+            env['LDFLAGS'] = '-L{}/usr/lib64 -L{}/usr/lib {}'.format(
+                            deps_destdir, deps_destdir, env.get('LDFLAGS', ''))
 
         try:
-            status, stdout, stderr = run_subprocess(str(pypy_c), args,
+            status, bld_stdout, bld_stderr = run_subprocess(str(pypy_c), args,
                                                     cwd=cwd, env=env)
             if status != 0:
                 print("stdout:")
-                print(stdout, file=sys.stderr)
+                print(bld_stdout, file=sys.stderr)
                 print("stderr:")
-                print(stderr, file=sys.stderr)
+                print(bld_stderr, file=sys.stderr)
                 raise RuntimeError('building {} failed'.format(key))
         except:
             import traceback;traceback.print_exc()
@@ -238,9 +254,13 @@ def create_cffi_import_libraries(pypy_c, options, basedir, only=None,
                          env=env)
             if status != 0:
                 failures.append((key, module))
-                print("stdout:")
+                print("build stdout:")
+                print(bld_stdout, file=sys.stderr)
+                print("build stderr:")
+                print(bld_stderr, file=sys.stderr)
+                print("test stdout:")
                 print(stdout, file=sys.stderr)
-                print("stderr:")
+                print("test stderr:")
                 print(stderr, file=sys.stderr)
         if os.path.exists(deps_destdir):
             shutil.rmtree(deps_destdir, ignore_errors=True)
