@@ -125,9 +125,9 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
     # checks for checking interp2app func argument names wrt unwrap_spec
     # and synthetizing an app-level signature
 
-    def __init__(self, original_sig):
-        self.func = original_sig.func
-        self.orig_arg = iter(original_sig.argnames).next
+    def __init__(self, func, argnames):
+        self.func = func
+        self.orig_arg = iter(argnames).next
 
     def visit_self(self, cls, app_sig):
         self.visit__W_Root(cls, app_sig)
@@ -160,6 +160,9 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
     def visit_text0(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
+    def visit_utf8(self, el, app_sig):
+        self.checked_space_method(el, app_sig)
+
     def visit_fsencode(self, el, app_sig):
         self.checked_space_method(el, app_sig)
 
@@ -304,6 +307,9 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
     def visit_text0(self, typ):
         self.run_args.append("space.text0_w(%s)" % (self.scopenext(),))
 
+    def visit_utf8(self, typ):
+        self.run_args.append("space.utf8_w(%s)" % (self.scopenext(),))
+
     def visit_fsencode(self, typ):
         self.run_args.append("space.fsencode_w(%s)" % (self.scopenext(),))
 
@@ -469,6 +475,9 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
     def visit_text0(self, typ):
         self.unwrap.append("space.text0_w(%s)" % (self.nextarg(),))
 
+    def visit_utf8(self, typ):
+        self.unwrap.append("space.utf8_w(%s)" % (self.nextarg(),))
+
     def visit_fsencode(self, typ):
         self.unwrap.append("space.fsencode_w(%s)" % (self.nextarg(),))
 
@@ -533,10 +542,10 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
 
 
 def int_unwrapping_space_method(typ):
-    assert typ in (int, str, float, unicode, r_longlong, r_uint, r_ulonglong, bool)
+    assert typ in (int, str, float, r_longlong, r_uint, r_ulonglong, bool)
     if typ is r_int is r_longlong:
         return 'gateway_r_longlong_w'
-    elif typ in (str, unicode, bool):
+    elif typ in (str, bool):
         return typ.__name__ + '_w'
     else:
         return 'gateway_' + typ.__name__ + '_w'
@@ -624,10 +633,15 @@ class BuiltinCode(Code):
     @not_rpython
     def __init__(self, func, unwrap_spec=None, self_type=None,
                  descrmismatch=None, doc=None):
+        from rpython.rlib import rutf8
+        from rpython.flowspace.bytecode import cpython_code_signature
         # 'implfunc' is the interpreter-level function.
         # Note that this uses a lot of (construction-time) introspection.
         Code.__init__(self, func.__name__)
         self.docstring = doc or func.__doc__
+        if self.docstring:
+            # check that it's utf-8
+            rutf8.check_utf8(self.docstring, False)
 
         self.identifier = "%s-%s-%s" % (func.__module__, func.__name__,
                                         getattr(self_type, '__name__', '*'))
@@ -644,8 +658,7 @@ class BuiltinCode(Code):
         # (function, cls) use function to check/unwrap argument of type cls
 
         # First extract the signature from the (CPython-level) code object
-        from pypy.interpreter import pycode
-        sig = pycode.cpython_code_signature(func.func_code)
+        sig = cpython_code_signature(func.func_code)
         argnames = sig.argnames
         varargname = sig.varargname
         kwargname = sig.kwargname
@@ -668,10 +681,9 @@ class BuiltinCode(Code):
             assert descrmismatch is None, (
                 "descrmismatch without a self-type specified")
 
-        orig_sig = SignatureBuilder(func, argnames, varargname, kwargname)
         app_sig = SignatureBuilder(func)
 
-        UnwrapSpec_Check(orig_sig).apply_over(unwrap_spec, app_sig)
+        UnwrapSpec_Check(func, argnames).apply_over(unwrap_spec, app_sig)
         self.sig = app_sig.signature()
         argnames = self.sig.argnames
         varargname = self.sig.varargname
@@ -700,6 +712,7 @@ class BuiltinCode(Code):
                     self.func__args__ = func
                 elif unwrap_spec == [self_type, ObjSpace, Arguments]:
                     self.__class__ = BuiltinCodePassThroughArguments1
+                    self.descr_reqcls = self_type
                     miniglobals = {'func': func, 'self_type': self_type}
                     d = {}
                     source = """if 1:
@@ -745,10 +758,7 @@ class BuiltinCode(Code):
         except DescrMismatch:
             if w_obj is not None:
                 args = args.prepend(w_obj)
-            return scope_w[0].descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args)
+            return self._type_unwrap_mismatch(space, args)
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -756,6 +766,15 @@ class BuiltinCode(Code):
             w_result = space.w_None
         return w_result
 
+    def _type_unwrap_mismatch(self, space, args):
+        w_obj = args.firstarg()
+        if w_obj is None:
+            raise oefmt(space.w_SystemError, "unexpected DescrMismatch error")
+        return w_obj.descr_call_mismatch(space,
+                                         self.descrmismatch_op,
+                                         self.descr_reqcls,
+                                         args)
+
     def handle_exception(self, space, e):
         try:
             if not we_are_translated():
@@ -778,10 +797,7 @@ class BuiltinCodePassThroughArguments0(BuiltinCode):
         try:
             w_result = self.func__args__(space, args)
         except DescrMismatch:
-            return args.firstarg().descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args)
+            return self._type_unwrap_mismatch(space, args)
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -799,10 +815,7 @@ class BuiltinCodePassThroughArguments1(BuiltinCode):
         try:
             w_result = self.func__args__(space, w_obj, args)
         except DescrMismatch:
-            return args.firstarg().descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args.prepend(w_obj))
+            return self._type_unwrap_mismatch(space, args.prepend(w_obj))
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -842,9 +855,7 @@ class BuiltinCode1(BuiltinCode):
         try:
             w_result = self.fastfunc_1(space, w1)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -868,9 +879,7 @@ class BuiltinCode2(BuiltinCode):
         try:
             w_result = self.fastfunc_2(space, w1, w2)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1, w2]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -895,9 +904,7 @@ class BuiltinCode3(BuiltinCode):
         try:
             w_result = self.fastfunc_3(space, w1, w2, w3)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1, w2, w3]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -923,9 +930,7 @@ class BuiltinCode4(BuiltinCode):
         try:
             w_result = self.fastfunc_4(space, w1, w2, w3, w4)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space,
                                                     [w1, w2, w3, w4]))
         except Exception as e:
@@ -959,8 +964,9 @@ def interpindirect2app(unbound_meth, unwrap_spec=None):
     else:
         assert isinstance(unwrap_spec, dict)
         unwrap_spec = unwrap_spec.copy()
-    unwrap_spec['self'] = base_cls
-    return interp2app(globals()['unwrap_spec'](**unwrap_spec)(f))
+    unwrap_spec['self'] = 'self'
+    return interp2app(globals()['unwrap_spec'](**unwrap_spec)(f),
+                      self_type=base_cls)
 
 class interp2app(W_Root):
     """Build a gateway that calls 'f' at interp-level."""
@@ -971,11 +977,11 @@ class interp2app(W_Root):
 
     @not_rpython
     def __new__(cls, f, app_name=None, unwrap_spec=None, descrmismatch=None,
-                as_classmethod=False, doc=None):
+                as_classmethod=False, doc=None, self_type=None):
 
         # f must be a function whose name does NOT start with 'app_'
-        self_type = None
         if hasattr(f, 'im_func'):
+            assert self_type in (None, f.im_class)
             self_type = f.im_class
             f = f.im_func
         if not isinstance(f, types.FunctionType):
