@@ -35,8 +35,7 @@ cpython_magic, = struct.unpack("<i", imp.get_magic())   # host magic number
 default_magic = (0xf303 + 7) | 0x0a0d0000               # this PyPy's magic
                                                         # (from CPython 2.7.0)
 
-# cpython_code_signature helper
-def cpython_code_signature(code):
+def make_signature(code):
     """Return a Signature instance."""
     argcount = code.co_argcount
     varnames = code.co_varnames
@@ -94,7 +93,7 @@ class PyCode(eval.Code):
         self.w_globals = None
         self.hidden_applevel = hidden_applevel
         self.magic = magic
-        self._signature = cpython_code_signature(self)
+        self._signature = make_signature(self)
         self._initialize()
         self._init_ready()
         self.new_code_hook()
