@@ -77,8 +77,7 @@ def generate_tokens(lines, flags):
     lnum = continued = 0
     namechars = NAMECHARS
     numchars = NUMCHARS
-    contstr, needcont = '', 0
-    contline = None
+    contstrs, needcont = [], False
     indents = [0]
     last_comment = ''
     parenstack = []
@@ -95,7 +94,7 @@ def generate_tokens(lines, flags):
         line = universal_newline(line)
         pos, max = 0, len(line)
 
-        if contstr:
+        if contstrs:
             if not line:
                 raise TokenError(
                     "end of file (EOF) while scanning triple-quoted string literal",
@@ -104,24 +103,23 @@ def generate_tokens(lines, flags):
             endmatch = endDFA.recognize(line)
             if endmatch >= 0:
                 pos = end = endmatch
-                tok = Token(tokens.STRING, contstr + line[:end], strstart[0],
+                contstrs.append(line[:end])
+                tok = Token(tokens.STRING, "".join(contstrs), strstart[0],
                        strstart[1], line)
                 token_list.append(tok)
                 last_comment = ''
-                contstr, needcont = '', 0
-                contline = None
+                contstrs, needcont = [], False
             elif (needcont and not line.endswith('\\\n') and
                                not line.endswith('\\\r\n')):
-                tok = Token(tokens.ERRORTOKEN, contstr + line, strstart[0],
+                contstrs.append(line)
+                tok = Token(tokens.ERRORTOKEN, "".join(contstrs), strstart[0],
                        strstart[1], line)
                 token_list.append(tok)
                 last_comment = ''
-                contstr = ''
-                contline = None
+                contstrs = []
                 continue
             else:
-                contstr = contstr + line
-                contline = contline + line
+                contstrs.append(line)
                 continue
 
         elif not parenstack and not continued:  # new statement
@@ -163,9 +161,9 @@ def generate_tokens(lines, flags):
 
         while pos < max:
             pseudomatch = pseudoDFA.recognize(line, pos)
+            start = whiteSpaceDFA.recognize(line, pos)
             if pseudomatch >= 0:                            # scan for tokens
                 # JDR: Modified
-                start = whiteSpaceDFA.recognize(line, pos)
                 if start < 0:
                     start = pos
                 end = pseudomatch
@@ -199,8 +197,7 @@ def generate_tokens(lines, flags):
                         last_comment = ''
                     else:
                         strstart = (lnum, start, line)
-                        contstr = line[start:]
-                        contline = line
+                        contstrs = [line[start:]]
                         break
                 elif initial in single_quoted or \
                     token[:2] in single_quoted or \
@@ -209,8 +206,7 @@ def generate_tokens(lines, flags):
                         strstart = (lnum, start, line)
                         endDFA = (endDFAs[initial] or endDFAs[token[1]] or
                                    endDFAs[token[2]])
-                        contstr, needcont = line[start:], 1
-                        contline = line
+                        contstrs, needcont = [line[start:]], True
                         break
                     else:                                  # ordinary string
                         tok = Token(tokens.STRING, token, lnum, start, line)
@@ -250,7 +246,6 @@ def generate_tokens(lines, flags):
                     token_list.append(Token(punct, token, lnum, start, line))
                     last_comment = ''
             else:
-                start = whiteSpaceDFA.recognize(line, pos)
                 if start < 0:
                     start = pos
                 if start<max and line[start] in single_quoted:
