@@ -10,6 +10,7 @@ from pypy.objspace.std.setobject import W_BaseSetObject
 from pypy.objspace.std.typeobject import MethodCache
 from pypy.objspace.std.mapdict import MapAttrCache
 from rpython.rlib import rposix, rgc, rstack
+from rpython.rtyper.lltypesystem import rffi
 
 
 def internal_repr(space, w_object):
@@ -86,14 +87,17 @@ def lookup_special(space, w_obj, meth):
         return space.w_None
     return space.get(w_descr, w_obj)
 
-def do_what_I_mean(space, w_crash=None):
-    if not space.is_none(w_crash):
-        raise ValueError    # RPython-level, uncaught
+def do_what_I_mean(space):
+    "Return 42"
     return space.newint(42)
 
+def _internal_crash(space, w_crash=None):
+    """for testing purposes, raise an interpreter-level ValueError. Should turn
+    into a SystemError automatically"""
+    raise ValueError    # RPython-level, uncaught
 
 def strategy(space, w_obj):
-    """ strategy(dict or list or set)
+    """ strategy(dict or list or set or instance)
 
     Return the underlying strategy currently used by a dict, list or set object
     """
@@ -104,9 +108,19 @@ def strategy(space, w_obj):
     elif isinstance(w_obj, W_BaseSetObject):
         name = w_obj.strategy.__class__.__name__
     else:
-        raise oefmt(space.w_TypeError, "expecting dict or list or set object")
+        m = w_obj._get_mapdict_map()
+        if m is not None:
+            name = m.repr()
+        else:
+            raise oefmt(space.w_TypeError, "expecting dict or list or set object, or instance of some kind")
     return space.newtext(name)
 
+def list_get_physical_size(space, w_obj):
+    if not isinstance(w_obj, W_ListObject):
+        raise oefmt(space.w_TypeError, "expected list")
+    return space.newint(w_obj.physical_size())
+
+
 def get_console_cp(space):
     """get_console_cp()
 
@@ -118,6 +132,19 @@ def get_console_cp(space):
         space.newtext('cp%d' % rwin32.GetConsoleOutputCP()),
         ])
 
+@unwrap_spec(fd=int)
+def get_osfhandle(space, fd):
+    """get_osfhandle()
+
+    Return the handle corresponding to the file descriptor (windows only)
+    """
+    from rpython.rlib import rwin32    # Windows only
+    try:
+        ret = rwin32.get_osfhandle(fd)
+        return space.newint(rffi.cast(rffi.INT, ret))
+    except OSError as e:
+        raise wrap_oserror(space, e)
+
 @unwrap_spec(sizehint=int)
 def resizelist_hint(space, w_list, sizehint):
     """ Reallocate the underlying storage of the argument list to sizehint """
