@@ -6,7 +6,7 @@ from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from rpython.rlib import jit, rgc
 from rpython.rtyper.lltypesystem import lltype, rffi
 
-from pypy.module._cffi_backend import get_dict_rtld_constants
+from pypy.module._cffi_backend.moduledef import get_dict_rtld_constants
 from pypy.module._cffi_backend import parse_c_type, realize_c_type
 from pypy.module._cffi_backend import newtype, cerrno, ccallback, ctypearray
 from pypy.module._cffi_backend import ctypestruct, ctypeptr, handle
@@ -558,13 +558,14 @@ Return the size in bytes of the argument.
 It can be a string naming a C type, or a 'cdata' instance."""
         #
         if isinstance(w_arg, W_CData):
+            w_ctype = w_arg.ctype
             size = w_arg._sizeof()
         else:
             w_ctype = self.ffi_type(w_arg, ACCEPT_ALL)
             size = w_ctype.size
-            if size < 0:
-                raise oefmt(self.w_FFIError,
-                            "don't know the size of ctype '%s'", w_ctype.name)
+        if size < 0:
+            raise oefmt(self.w_FFIError,
+                        "don't know the size of ctype '%s'", w_ctype.name)
         return self.space.newint(size)
 
 
@@ -761,7 +762,7 @@ if sys.platform == 'win32':
 
 
 W_FFIObject.typedef = TypeDef(
-        'CompiledFFI',
+        '_cffi_backend.FFI',
         __new__     = interp2app(W_FFIObject___new__),
         __init__    = interp2app(W_FFIObject.descr_init),
         CData       = ClassAttr(make_CData),
