@@ -18,8 +18,15 @@ class AppTestSemaphore:
         cls.w_SEMAPHORE = cls.space.wrap(SEMAPHORE)
         cls.w_RECURSIVE = cls.space.wrap(RECURSIVE_MUTEX)
         cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
+        # import here since importing _multiprocessing imports multiprocessing
+        # (in interp_connection) to get the BufferTooShort exception, which on
+        # win32 imports msvcrt which imports via cffi which allocates ccharp
+        # that are never released. This trips up the LeakChecker if done in a
+        # test function
+        cls.w_multiprocessing = cls.space.appexec([],
+                                  '(): import multiprocessing as m; return m')
 
-    def test_semaphore(self):
+    def test_semaphore_basic(self):
         from _multiprocessing import SemLock
         import sys
         assert SemLock.SEM_VALUE_MAX > 10
@@ -77,6 +84,31 @@ class AppTestSemaphore:
         sem.release()
         sem.release()
 
+    def test_semaphore_maxvalue(self):
+        from _multiprocessing import SemLock
+        import sys
+        kind = self.SEMAPHORE
+        value = SemLock.SEM_VALUE_MAX
+        maxvalue = SemLock.SEM_VALUE_MAX
+        sem = SemLock(kind, value, maxvalue)
+
+        for i in range(10):
+            res = sem.acquire()
+            assert res == True
+            assert sem._count() == i+1
+            if sys.platform != 'darwin':
+                assert sem._get_value() == maxvalue - (i+1)
+
+        value = 0
+        maxvalue = SemLock.SEM_VALUE_MAX
+        sem = SemLock(kind, value, maxvalue)
+
+        for i in range(10):
+            sem.release()
+            assert sem._count() == -(i+1)
+            if sys.platform != 'darwin':
+                assert sem._get_value() == i+1
+
     def test_semaphore_wait(self):
         from _multiprocessing import SemLock
         kind = self.SEMAPHORE
