@@ -45,7 +45,7 @@ def callback(ll_args, ll_res, ll_userdata):
                     space, rffi.cast(rffi.SIZE_T, ll_args[i]))
             else:
                 # XXX other types?
-                args_w[i] = space.newint(rffi.cast(rffi.ULONG, ll_args[i]))
+                args_w[i] = space.newint(rffi.cast(lltype.Unsigned, ll_args[i]))
         w_res = space.call(w_callable, space.newtuple(args_w))
         if callback_ptr.result is not None: # don't return void
             ptr = ll_res
