@@ -245,6 +245,7 @@ class AppTestFfi:
         cls.w_sizes_and_alignments = space.wrap(dict(
             [(k, (v.c_size, v.c_alignment)) for k,v in TYPEMAP.iteritems()]))
         cls.w_float_typemap = space.wrap(TYPEMAP_FLOAT_LETTERS)
+        cls.w_is64bit = space.wrap(sys.maxint > 2147483647)
 
     def test_libload(self):
         import _rawffi
@@ -719,7 +720,7 @@ class AppTestFfi:
         ll_to_sort = _rawffi.Array('i')(4)
         for i in range(4):
             ll_to_sort[i] = 4-i
-        qsort = libc.ptr('qsort', ['P', 'l', 'l', 'P'], None)
+        qsort = libc.ptr('qsort', ['P', 'Z', 'Z', 'P'], None)
         bogus_args = []
         def compare(a, b):
             a1 = _rawffi.Array('i').fromaddress(_rawffi.Array('P').fromaddress(a, 1)[0], 1)
@@ -730,11 +731,11 @@ class AppTestFfi:
                 return 1
             return -1
         a1 = ll_to_sort.byptr()
-        a2 = _rawffi.Array('l')(1)
+        a2 = _rawffi.Array('Z')(1)
         a2[0] = len(ll_to_sort)
-        a3 = _rawffi.Array('l')(1)
+        a3 = _rawffi.Array('Z')(1)
         a3[0] = struct.calcsize('i')
-        cb = _rawffi.CallbackPtr(compare, ['P', 'P'], 'l')
+        cb = _rawffi.CallbackPtr(compare, ['P', 'P'], 'i')
         a4 = cb.byptr()
         qsort(a1, a2, a3, a4)
         res = [ll_to_sort[i] for i in range(len(ll_to_sort))]
@@ -1022,8 +1023,8 @@ class AppTestFfi:
         raises(_rawffi.SegfaultException, a.__setitem__, 3, 3)
 
     def test_stackcheck(self):
-        if self.platform != "msvc":
-            skip("win32 msvc specific")
+        if self.platform != "msvc" or self.is64bit:
+            skip("32-bit win32 msvc specific")
 
         # Even if the call corresponds to the specified signature,
         # the STDCALL calling convention may detect some errors
