@@ -1,3 +1,6 @@
+
+from rpython.rlib import rutf8
+
 from pypy.interpreter.gateway import unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError, oefmt
 
@@ -208,10 +211,11 @@ def show_warning(space, w_filename, lineno, w_text, w_category,
     except OperationError as e:
         if e.async(space):
             raise
-        message = u"%s:%d: %s: %s\n" % (space.unicode_w(w_filename), lineno,
-                                        space.unicode_w(w_name),
-                                        space.unicode_w(w_text))
-        w_message = space.newunicode(message)
+        message = "%s:%d: %s: %s\n" % (space.utf8_w(w_filename), lineno,
+                                        space.utf8_w(w_name),
+                                        space.utf8_w(w_text))
+        lgt = rutf8.check_utf8(message, True)
+        w_message = space.newutf8(message, lgt)
     else:
         w_message = space.newtext(message)
     space.call_method(w_stderr, "write", w_message)
@@ -282,15 +286,17 @@ def do_warn_explicit(space, w_category, w_message, context_w,
     if action == "error":
         raise OperationError(w_category, w_message)
 
+    if action == 'ignore':
+        return
+
     # Store in the registry that we've been here, *except* when the action is
     # "always".
     warned = False
     if action != 'always':
         if not space.is_w(w_registry, space.w_None):
             space.setitem(w_registry, w_key, space.w_True)
-        if action == 'ignore':
-            return
-        elif action == 'once':
+
+        if action == 'once':
             if space.is_w(w_registry, space.w_None):
                 w_registry = get_once_registry(space)
             warned = update_registry(space, w_registry, w_text, w_category)
@@ -350,7 +356,7 @@ def get_source_line(space, w_globals, lineno):
         return None
 
     # Split the source into lines.
-    w_source_list = space.call_method(w_source, "splitlines")
+    w_source_list = space.call_method(space.w_text, "splitlines", w_source)
 
     # Get the source line.
     w_source_line = space.getitem(w_source_list, space.newint(lineno - 1))
