@@ -156,6 +156,8 @@ class WeakrefLifeline(W_Root):
 
 
 class W_WeakrefBase(W_Root):
+    exact_class_applevel_name = 'weakref-or-proxy'
+
     def __init__(self, space, w_obj, w_callable):
         assert w_callable is not space.w_None    # should be really None
         self.space = space
@@ -193,6 +195,15 @@ class W_Weakref(W_WeakrefBase):
         W_WeakrefBase.__init__(self, space, w_obj, w_callable)
         self.w_hash = None
 
+    def _cleanup_(self):
+        # When a prebuilt weakref is frozen inside a translation, if
+        # this weakref has got an already-cached w_hash, then throw it
+        # away.  That's because the hash value will change after
+        # translation.  It will be recomputed the first time we ask for
+        # it.  Note that such a frozen weakref, if not dead, will point
+        # to a frozen object, so it will never die.
+        self.w_hash = None
+
     def descr__init__weakref(self, space, w_obj, w_callable=None,
                              __args__=None):
         if __args__.arguments_w:
@@ -217,7 +228,7 @@ class W_Weakref(W_WeakrefBase):
             return self.space.w_None
         return w_obj
 
-    def descr__eq__(self, space, w_ref2):
+    def compare(self, space, w_ref2, invert):
         if not isinstance(w_ref2, W_Weakref):
             return space.w_NotImplemented
         ref1 = self
@@ -225,11 +236,18 @@ class W_Weakref(W_WeakrefBase):
         w_obj1 = ref1.dereference()
         w_obj2 = ref2.dereference()
         if w_obj1 is None or w_obj2 is None:
-            return space.is_(ref1, ref2)
-        return space.eq(w_obj1, w_obj2)
+            w_res = space.is_(ref1, ref2)
+        else:
+            w_res = space.eq(w_obj1, w_obj2)
+        if invert:
+            w_res = space.not_(w_res)
+        return w_res
+
+    def descr__eq__(self, space, w_ref2):
+        return self.compare(space, w_ref2, invert=False)
 
     def descr__ne__(self, space, w_ref2):
-        return space.not_(space.eq(self, w_ref2))
+        return self.compare(space, w_ref2, invert=True)
 
 def getlifeline(space, w_obj):
     lifeline = w_obj.getweakref()
@@ -271,10 +289,10 @@ def getweakrefcount(space, w_obj):
     """Return the number of weak references to 'obj'."""
     lifeline = w_obj.getweakref()
     if lifeline is None:
-        return space.wrap(0)
+        return space.newint(0)
     else:
         result = lifeline.traverse(_weakref_count, 0)
-        return space.wrap(result)
+        return space.newint(result)
 
 def _get_weakrefs(lifeline, wref, result):
     w_ref = wref()
@@ -350,9 +368,26 @@ for opname, _, arity, special_methods in ObjSpace.MethodTable:
     exec py.code.Source(code).compile()
 
     func.func_name = opname
-    for special_method in special_methods:
-        proxy_typedef_dict[special_method] = interp2app(func)
-        callable_proxy_typedef_dict[special_method] = interp2app(func)
+    if len(special_methods) == 2 and special_methods[1].startswith("__r"):
+        proxy_typedef_dict[special_methods[0]] = interp2app(func)
+        callable_proxy_typedef_dict[special_methods[0]] = interp2app(func)
+
+        # HACK: need to call the space method with arguments in the reverse
+        # order!
+        code = code.replace("(w_obj0, w_obj1)", "(w_obj1, w_obj0)")
+        code = code.replace("func", "rfunc")
+
+        exec py.code.Source(code).compile()
+        rfunc.func_name = special_methods[1][2:-2]
+
+        proxy_typedef_dict[special_methods[1]] = interp2app(rfunc)
+        callable_proxy_typedef_dict[special_methods[1]] = interp2app(rfunc)
+    elif opname in ["lt", "le", "gt", "ge", "eq", "ne"]:
+        proxy_typedef_dict[special_methods[0]] = interp2app(func)
+    else:
+        for special_method in special_methods:
+            proxy_typedef_dict[special_method] = interp2app(func)
+            callable_proxy_typedef_dict[special_method] = interp2app(func)
 
 # __unicode__ is not yet a space operation
 def proxy_unicode(space, w_obj):
