@@ -1,6 +1,6 @@
 from pypy.module.cpyext.api import (
     cpython_api, CANNOT_FAIL, cpython_struct)
-from pypy.module.cpyext.pyobject import PyObject, Py_DecRef, make_ref
+from pypy.module.cpyext.pyobject import PyObject, decref, make_ref
 from pypy.interpreter.error import OperationError
 from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rlib import rthread
@@ -75,7 +75,7 @@ def encapsulator(T, flavor='raw', dealloc=None):
 
 def ThreadState_dealloc(ts, space):
     assert space is not None
-    Py_DecRef(space, ts.c_dict)
+    decref(space, ts.c_dict)
 ThreadStateCapsule = encapsulator(PyThreadState.TO,
                                   dealloc=ThreadState_dealloc)
 
@@ -248,6 +248,11 @@ def _workaround_cpython_untranslated(space):
     space.threadlocals.get_ec = get_possibly_deleted_ec
 
 
+@cpython_api([], rffi.INT_real, error=CANNOT_FAIL, gil="pygilstate_check")
+def PyGILState_Check(space):
+    assert False, "the logic is completely inside wrapper_second_level"
+
+
 @cpython_api([], PyGILState_STATE, error=CANNOT_FAIL, gil="pygilstate_ensure")
 def PyGILState_Ensure(space, previous_state):
     # The argument 'previous_state' is not part of the API; it is inserted
@@ -319,7 +324,7 @@ def PyThreadState_Clear(space, tstate):
     interpreter lock must be held."""
     if not space.config.translation.thread:
         raise NoThreads
-    Py_DecRef(space, tstate.c_dict)
+    decref(space, tstate.c_dict)
     tstate.c_dict = lltype.nullptr(PyObject.TO)
     space.threadlocals.leave_thread(space)
     space.getexecutioncontext().cleanup_cpyext_state()
