@@ -1,6 +1,5 @@
-import pytest
-import struct
 import sys
+import pytest
 from pypy.interpreter.error import OperationError
 from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rlib.rarithmetic import maxint
@@ -11,36 +10,31 @@ from pypy.module.cpyext.longobject import (PyLong_Check, PyLong_CheckExact,
     PyLong_FromLong, PyLong_AsLong, PyLong_AsUnsignedLong, PyLong_AsLongLong,
     PyLong_AsUnsignedLongLong, PyLong_AsUnsignedLongLongMask)
 
-maxlong = int(2 ** (struct.calcsize('l') * 8 - 1) - 1)
-
 class TestLongObject(BaseApiTest):
     def test_FromLong(self, space, api):
         w_value = api.PyLong_FromLong(3)
         assert isinstance(w_value, W_LongObject)
         assert space.unwrap(w_value) == 3
 
-        w_value = api.PyLong_FromLong(maxlong)
+        w_value = api.PyLong_FromLong(sys.maxint)
         assert isinstance(w_value, W_LongObject)
-        assert space.unwrap(w_value) == maxlong
+        assert space.unwrap(w_value) == sys.maxint
 
     def test_aslong(self, space):
-        w_value = PyLong_FromLong(space, (maxlong - 1) // 2)
+        w_value = PyLong_FromLong(space, (sys.maxint - 1) / 2)
         assert isinstance(w_value, W_LongObject)
 
         w_value = space.mul(w_value, space.wrap(2))
         assert isinstance(w_value, W_LongObject)
         value = PyLong_AsLong(space, w_value)
-        assert value == (maxlong - 1)
+        assert value == (sys.maxint - 1)
 
         w_value = space.mul(w_value, space.wrap(2))
         with pytest.raises(OperationError) as excinfo:
             PyLong_AsLong(space, w_value)
         assert excinfo.value.w_type is space.w_OverflowError
-        msg = "Python int too large to convert to C long"
-        print(space.text_w(excinfo.value.get_w_value(space)))
-        assert space.text_w(excinfo.value.get_w_value(space)) == msg
         value = PyLong_AsUnsignedLong(space, w_value)
-        assert value == (maxlong - 1) * 2
+        assert value == (sys.maxint - 1) * 2
 
         with pytest.raises(OperationError) as excinfo:
             PyLong_AsUnsignedLong(space, space.newint(-1))
@@ -57,7 +51,7 @@ class TestLongObject(BaseApiTest):
 
     def test_fromdouble(self, space, api):
         w_value = api.PyLong_FromDouble(-12.74)
-        assert space.isinstance_w(w_value, space.w_int)
+        assert isinstance(w_value, W_LongObject)
         assert space.unwrap(w_value) == -12
         assert api.PyLong_AsDouble(w_value) == -12
 
@@ -67,11 +61,11 @@ class TestLongObject(BaseApiTest):
         assert PyLong_CheckExact(space, w_l)
 
         w_i = space.wrap(sys.maxint)
-        assert PyLong_Check(space, w_i)
-        assert PyLong_CheckExact(space, w_i)
+        assert not PyLong_Check(space, w_i)
+        assert not PyLong_CheckExact(space, w_i)
 
         L = space.appexec([], """():
-            class L(int):
+            class L(long):
                 pass
             return L
         """)
@@ -86,32 +80,32 @@ class TestLongObject(BaseApiTest):
         assert excinfo.value.w_type is space.w_OverflowError
 
         assert PyLong_AsUnsignedLongLong(space, space.wrap(1 << 63)) == 1 << 63
+
         with pytest.raises(OperationError) as excinfo:
             PyLong_AsUnsignedLongLong(space, space.wrap(1 << 64))
         assert excinfo.value.w_type is space.w_OverflowError
 
+        with pytest.raises(OperationError) as excinfo:
+            PyLong_AsUnsignedLongLong(space, None)
+        assert excinfo.value.w_type is space.w_SystemError
+
         assert PyLong_AsUnsignedLongLongMask(space, space.wrap(1 << 64)) == 0
 
         with pytest.raises(OperationError) as excinfo:
             PyLong_AsUnsignedLongLong(space, space.newint(-1))
         assert excinfo.value.w_type is space.w_OverflowError
+
         with pytest.raises(OperationError) as excinfo:
             PyLong_AsUnsignedLongLongMask(space, None)
         assert excinfo.value.w_type is space.w_SystemError
 
+
     def test_as_long_and_overflow(self, space, api):
         overflow = lltype.malloc(rffi.CArrayPtr(rffi.INT_real).TO, 1, flavor='raw')
-        overflow[0] = rffi.cast(rffi.INT_real, 123)
         assert api.PyLong_AsLongAndOverflow(
-            space.wrap(maxlong), overflow) == maxlong
-        assert not api.PyErr_Occurred()
-        assert overflow[0] == 0
+            space.wrap(sys.maxint), overflow) == sys.maxint
         assert api.PyLong_AsLongAndOverflow(
-            space.wrap(maxlong + 1), overflow) == -1
-        assert not api.PyErr_Occurred()
-        assert overflow[0] == 1
-        assert api.PyLong_AsLongAndOverflow(
-            space.wrap(-maxlong - 2), overflow) == -1
+            space.wrap(-sys.maxint - 2), overflow) == -1
         assert not api.PyErr_Occurred()
         assert overflow[0] == -1
         lltype.free(overflow, flavor='raw')
@@ -131,20 +125,24 @@ class TestLongObject(BaseApiTest):
         lltype.free(overflow, flavor='raw')
 
     def test_as_voidptr(self, space, api):
+        # CPython returns an int (not a long) depending on the value
+        # passed to PyLong_FromVoidPtr().  In all cases, NULL becomes
+        # the int 0.
         w_l = api.PyLong_FromVoidPtr(lltype.nullptr(rffi.VOIDP.TO))
         assert space.is_w(space.type(w_l), space.w_int)
         assert space.unwrap(w_l) == 0
         assert api.PyLong_AsVoidPtr(w_l) == lltype.nullptr(rffi.VOIDP.TO)
-
+        # Positive values also return an int (assuming, like always in
+        # PyPy, that an int is big enough to store any pointer).
         p = rffi.cast(rffi.VOIDP, maxint)
         w_l = api.PyLong_FromVoidPtr(p)
         assert space.is_w(space.type(w_l), space.w_int)
         assert space.unwrap(w_l) == maxint
         assert api.PyLong_AsVoidPtr(w_l) == p
-
+        # Negative values always return a long.
         p = rffi.cast(rffi.VOIDP, -maxint-1)
         w_l = api.PyLong_FromVoidPtr(p)
-        assert space.is_w(space.type(w_l), space.w_int)
+        assert space.is_w(space.type(w_l), space.w_long)
         assert space.unwrap(w_l) == maxint+1
         assert api.PyLong_AsVoidPtr(w_l) == p
 
@@ -152,9 +150,6 @@ class TestLongObject(BaseApiTest):
         assert api._PyLong_Sign(space.wraplong(0L)) == 0
         assert api._PyLong_Sign(space.wraplong(2L)) == 1
         assert api._PyLong_Sign(space.wraplong(-2L)) == -1
-        assert api._PyLong_Sign(space.wrap(0)) == 0
-        assert api._PyLong_Sign(space.wrap(42)) == 1
-        assert api._PyLong_Sign(space.wrap(-42)) == -1
 
         assert api._PyLong_NumBits(space.wrap(0)) == 0
         assert api._PyLong_NumBits(space.wrap(1)) == 1
@@ -164,12 +159,6 @@ class TestLongObject(BaseApiTest):
         assert api._PyLong_NumBits(space.wrap(3)) == 2
         assert api._PyLong_NumBits(space.wrap(-3)) == 2
 
-    def test_as_ulongmask(self, space, api):
-        assert api.PyLong_AsUnsignedLongMask(
-            space.wrap(maxlong * 2 + 1)) == maxlong * 2 + 1
-        assert api.PyLong_AsUnsignedLongMask(
-            space.wrap(maxlong * 2 + 2)) == 0
-
 class AppTestLongObject(AppTestCpythonExtensionBase):
     def test_fromunsignedlong(self):
         module = self.import_extension('foo', [
@@ -185,11 +174,9 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                     return NULL;
                  }
                  return obj;
-             """),
-            ])
-        import struct
-        max_ul = struct.unpack_from('L', b'\xff'*8)[0]
-        assert module.from_unsignedlong() == max_ul
+             """)])
+        import sys
+        assert module.from_unsignedlong() == 2 * sys.maxint + 1
 
     def test_fromlonglong(self):
         module = self.import_extension('foo', [
@@ -237,7 +224,7 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                  return PyLong_FromSize_t((size_t)-1);
              """)])
         import sys
-        assert module.from_unsignedlong() == 2 * sys.maxsize + 1
+        assert module.from_unsignedlong() == 2 * sys.maxint + 1
 
     def test_fromstring(self):
         module = self.import_extension('foo', [
@@ -255,7 +242,7 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                  int little_endian, is_signed;
                  if (!PyArg_ParseTuple(args, "ii", &little_endian, &is_signed))
                      return NULL;
-                 return _PyLong_FromByteArray((unsigned char*)"\\x9A\\xBC", 2,
+                 return _PyLong_FromByteArray((unsigned char*)"\x9A\xBC", 2,
                                               little_endian, is_signed);
              """),
             ])
@@ -271,7 +258,7 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                  int little_endian, is_signed;
                  if (!PyArg_ParseTuple(args, "ii", &little_endian, &is_signed))
                      return NULL;
-                 return _PyLong_FromByteArray((unsigned char*)"\\x9A\\xBC\\x41", 3,
+                 return _PyLong_FromByteArray((unsigned char*)"\x9A\xBC\x41", 3,
                                               little_endian, is_signed);
              """),
             ])
@@ -298,29 +285,29 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                      free(bytes);
                      return NULL;
                  }
-                 result = PyBytes_FromStringAndSize((const char *)bytes, n);
+                 result = PyString_FromStringAndSize((const char *)bytes, n);
                  free(bytes);
                  return result;
              """),
             ])
-        s = module.as_bytearray(0x41BC9A, 4, True, False)
-        assert s == b"\x9A\xBC\x41\x00"
-        s = module.as_bytearray(0x41BC9A, 4, False, False)
-        assert s == b"\x00\x41\xBC\x9A"
-        s = module.as_bytearray(0x41BC9A, 3, True, False)
-        assert s == b"\x9A\xBC\x41"
-        s = module.as_bytearray(0x41BC9A, 3, True, True)
-        assert s == b"\x9A\xBC\x41"
-        s = module.as_bytearray(0x9876, 2, True, False)
-        assert s == b"\x76\x98"
-        s = module.as_bytearray(0x9876 - 0x10000, 2, True, True)
-        assert s == b"\x76\x98"
+        s = module.as_bytearray(0x41BC9AL, 4, True, False)
+        assert s == "\x9A\xBC\x41\x00"
+        s = module.as_bytearray(0x41BC9AL, 4, False, False)
+        assert s == "\x00\x41\xBC\x9A"
+        s = module.as_bytearray(0x41BC9AL, 3, True, False)
+        assert s == "\x9A\xBC\x41"
+        s = module.as_bytearray(0x41BC9AL, 3, True, True)
+        assert s == "\x9A\xBC\x41"
+        s = module.as_bytearray(0x9876L, 2, True, False)
+        assert s == "\x76\x98"
+        s = module.as_bytearray(0x9876L - 0x10000L, 2, True, True)
+        assert s == "\x76\x98"
         raises(OverflowError, module.as_bytearray,
-                              0x9876, 2, False, True)
+                              0x9876L, 2, False, True)
         raises(OverflowError, module.as_bytearray,
-                              -1, 2, True, False)
+                              -1L, 2, True, False)
         raises(OverflowError, module.as_bytearray,
-                              0x1234567, 3, True, False)
+                              0x1234567L, 3, True, False)
 
     def test_fromunicode(self):
         module = self.import_extension('foo', [
@@ -335,61 +322,6 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
         # A string with arabic digits. 'BAD' is after the 6th character.
         assert module.from_unicode(u'  1\u0662\u0663\u0664BAD') == (1234, 4660)
 
-    def test_fromunicodeobject(self):
-        module = self.import_extension('foo', [
-            ("from_unicodeobject", "METH_O",
-             """
-                 return Py_BuildValue("NN",
-                     PyLong_FromUnicodeObject(args, 10),
-                     PyLong_FromUnicodeObject(args, 16));
-             """),
-            ])
-        # A string with arabic digits.
-        assert (module.from_unicodeobject(u'  1\u0662\u0663\u0664')
-                == (1234, 4660))
-
-    def test_aslong(self):
-        module = self.import_extension('foo', [
-            ("as_long", "METH_O",
-             """
-                long n = PyLong_AsLong(args);
-                if (n == -1 && PyErr_Occurred()) {
-                    return NULL;
-                }
-                return PyLong_FromLong(n);
-             """),
-            ("long_max", "METH_NOARGS",
-             """
-                return  PyLong_FromLong(LONG_MAX);
-             """
-            ),
-            ("long_min", "METH_NOARGS",
-             """
-                return  PyLong_FromLong(LONG_MIN);
-             """
-            ),
-            ])
-        assert module.as_long(123) == 123
-        assert module.as_long(-1) == -1
-        assert module.as_long(1.23) == 1
-        LONG_MAX = module.long_max()
-        LONG_MIN = module.long_min()
-        assert module.as_long(LONG_MAX) == LONG_MAX
-        raises(OverflowError, module.as_long, LONG_MAX+ 1)
-        assert module.as_long(LONG_MIN) == LONG_MIN
-        raises(OverflowError, module.as_long, LONG_MIN - 1)
-        class A:
-            def __index__(self):
-                return 42
-
-            def __int__(self):
-                return 21
-
-        a = A()
-        assert int(a) == 21
-        # new for python3.8: first try __index__
-        assert module.as_long(a) == 42
-
     def test_strtol(self):
         module = self.import_extension('foo', [
             ("from_str", "METH_NOARGS",
@@ -436,6 +368,28 @@ class AppTestLongObject(AppTestCpythonExtensionBase):
                 Py_DECREF(one);
                 Py_DECREF(obj);
                 return ret;
+             """),
+            ("has_hex", "METH_NOARGS",
+             """
+                PyObject *ret, *obj = PyLong_FromLong(42);
+                if (obj->ob_type->tp_as_number->nb_hex)
+                    ret = obj->ob_type->tp_as_number->nb_hex(obj);
+                else
+                    ret = PyLong_FromLong(-1);
+                Py_DECREF(obj);
+                return ret;
+             """),
+            ("has_oct", "METH_NOARGS",
+             """
+                PyObject *ret, *obj = PyLong_FromLong(42);
+                if (obj->ob_type->tp_as_number->nb_oct)
+                    ret = obj->ob_type->tp_as_number->nb_oct(obj);
+                else
+                    ret = PyLong_FromLong(-1);
+                Py_DECREF(obj);
+                return ret;
              """)])
         assert module.has_sub() == 0
         assert module.has_pow() == 0
+        assert module.has_hex() == '0x2aL'
+        assert module.has_oct() == '052L'
