@@ -214,9 +214,9 @@ class TestObject(BaseApiTest):
                 PyObject_Cmp(space, w(u"\xe9"), w("\xe9"), ptr)
 
     def test_unicode(self, space, api):
-        assert space.unicode_w(api.PyObject_Unicode(None)) == u"<NULL>"
-        assert space.unicode_w(api.PyObject_Unicode(space.wrap([]))) == u"[]"
-        assert space.unicode_w(api.PyObject_Unicode(space.wrap("e"))) == u"e"
+        assert space.utf8_w(api.PyObject_Unicode(None)) == u"<NULL>"
+        assert space.utf8_w(api.PyObject_Unicode(space.wrap([]))) == u"[]"
+        assert space.utf8_w(api.PyObject_Unicode(space.wrap("e"))) == u"e"
         with raises_w(space, UnicodeDecodeError):
             PyObject_Unicode(space, space.wrap("\xe9"))
 
@@ -377,6 +377,8 @@ class AppTestObject(AppTestCpythonExtensionBase):
             """)])
         a = module.empty_format('hello')
         assert isinstance(a, unicode)
+        a = module.empty_format(type('hello'))
+        assert isinstance(a, unicode)
 
     def test_add_memory_pressure(self):
         self.reset_memory_pressure()    # for the potential skip
