@@ -96,6 +96,7 @@ class TestMisc(BaseTestPyPyC):
             class A(object):
                 def __init__(self, val):
                     self.val1 = self.val2 = val
+            A("x") # prevent field unboxing
             a = A(1)
             b = A(1)
             sa = 0
@@ -113,6 +114,7 @@ class TestMisc(BaseTestPyPyC):
             i12 = int_is_true(i4)
             guard_true(i12, descr=...)
             guard_not_invalidated(descr=...)
+            guard_nonnull_class(p10, ConstClass(W_IntObject), descr=...)
             i10p = getfield_gc_i(p10, descr=...)
             i10 = int_mul_ovf(2, i10p)
             guard_no_overflow(descr=...)
@@ -148,6 +150,8 @@ class TestMisc(BaseTestPyPyC):
         setfield_gc(p9, i17, descr=<.* .*W_XRangeIterator.inst_current .*>)
         guard_not_invalidated(descr=...)
         i18 = force_token()
+        i83 = int_lt(0, i14)
+        guard_true(i83, descr=...)
         i84 = int_sub(i14, 1)
         i21 = int_lt(i10, 0)
         guard_false(i21, descr=...)
@@ -175,12 +179,16 @@ class TestMisc(BaseTestPyPyC):
         loop, = log.loops_by_filename(self.filepath)
         assert loop.match("""
             guard_not_invalidated?
+            i80 = int_lt(i11, 0)
+            guard_false(i80, descr=...)
             i16 = int_ge(i11, i12)
             guard_false(i16, descr=...)
             i20 = int_add(i11, 1)
             setfield_gc(p4, i20, descr=<.* .*W_AbstractSeqIterObject.inst_index .*>)
             guard_not_invalidated?
             i21 = force_token()
+            i89 = int_lt(0, i9)
+            guard_true(i89, descr=...)
             i88 = int_sub(i9, 1)
             i25 = int_ge(i11, i9)
             guard_false(i25, descr=...)
@@ -214,6 +222,8 @@ class TestMisc(BaseTestPyPyC):
             setfield_gc(p4, i20, descr=<.* .*W_AbstractSeqIterObject.inst_index .*>)
             guard_not_invalidated?
             i21 = force_token()
+            i94 = int_lt(0, i9)
+            guard_true(i94, descr=...)
             i95 = int_sub(i9, 1)
             i23 = int_lt(i18, 0)
             guard_false(i23, descr=...)
@@ -273,7 +283,7 @@ class TestMisc(BaseTestPyPyC):
             i21 = getfield_gc_i(p17, descr=<FieldS .*W_Array.*.inst_len .*>)
             i23 = int_lt(0, i21)
             guard_true(i23, descr=...)
-            i24 = getfield_gc_i(p17, descr=<FieldU .*W_ArrayTypei.inst_buffer .*>)
+            i24 = getfield_gc_i(p17, descr=<FieldU .*W_ArrayBase.inst__buffer .*>)
             i25 = getarrayitem_raw_i(i24, 0, descr=<.*>)
             i27 = int_lt(1, i21)
             guard_false(i27, descr=...)
@@ -387,7 +397,8 @@ class TestMisc(BaseTestPyPyC):
     def test_long_comparison(self):
         def main(n):
             while n:
-                12345L > 123L  # ID: long_op
+                x = 12345L
+                x > 123L  # ID: long_op
                 n -= 1
 
         log = self.run(main, [300])
