@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import sys
 from pypy.module.pypyjit.test_pypy_c.test_00_model import BaseTestPyPyC
 
@@ -23,22 +24,26 @@ class TestString(BaseTestPyPyC):
             guard_true(i14, descr=...)
             guard_not_invalidated(descr=...)
             i16 = int_eq(i6, %d)
-            i19 = call_i(ConstClass(ll_int_py_mod__Signed_Signed), i6, i10, descr=<Calli . ii EF=0 OS=14>)
-            i21 = int_lt(i19, 0)
+            i83 = call_i(ConstClass(ll_int_py_mod__Signed_Signed), i6, i10, descr=<Calli . ii EF=0 OS=14>)
+            i21 = int_lt(i83, 0)
             guard_false(i21, descr=...)
-            i22 = int_ge(i19, i10)
+            i22 = int_ge(i83, i10)
             guard_false(i22, descr=...)
-            i23 = strgetitem(p11, i19)
-            i24 = int_ge(i19, i12)
+            i89 = strgetitem(p55, i83)
+            i24 = int_ge(i83, i12)
             guard_false(i24, descr=...)
-            i25 = unicodegetitem(p13, i19)
-            p27 = newstr(1)
-            strsetitem(p27, 0, i23)
-            p30 = call_r(ConstClass(ll_str2unicode__rpy_stringPtr), p27, descr=...)
+            i93 = int_add(i83, 1)
+            i94 = int_gt(i93, i56)
+            guard_false(i94, descr=...)
+            p96 = newstr(1)
+            strsetitem(p96, 0, i89)
+            i98 = call_i(ConstClass(first_non_ascii_char), p96, descr=<Calli . r EF=4>)
             guard_no_exception(descr=...)
-            i32 = call_i(ConstClass(_ll_2_str_eq_checknull_char__rpy_unicodePtr_UniChar), p30, i25, descr=...)
-            guard_true(i32, descr=...)
-            i34 = int_add(i6, 1)
+            i100 = int_lt(i98, 0)
+            guard_true(i100, descr=...)
+            i102 = call_i(ConstClass(_ll_4_str_eq_slice_char__rpy_stringPtr_Signed_Signed_Char), p13, i83, 1, i89, descr=<Calli . riii EF=0 OS=27>)
+            guard_true(i102, descr=...)
+            i104 = int_add(i6, 1)
             --TICK--
             jump(..., descr=...)
         """ % (-sys.maxint-1,))
@@ -139,11 +144,11 @@ class TestString(BaseTestPyPyC):
             jump(..., descr=...)
         """)
 
-    def test_remove_duplicate_method_calls(self):
+    def test_remove_duplicate_method_calls_bytes(self):
         def main(n):
             lst = []
             for i in range(n):
-                s = 'Hello %d' % i
+                s = b'Hello %d' % i
                 t = s.lower()   # ID: callone
                 u = s.lower()   # ID: calltwo
                 lst.append(t)
@@ -159,6 +164,30 @@ class TestString(BaseTestPyPyC):
             ''')
         assert loop.match_by_id('calltwo', '')    # nothing
 
+    def test_remove_duplicate_method_calls_unicode(self):
+        def main(n):
+            lst = []
+            for i in range(n):
+                s = u'H\xe4llo %d' % i
+                t = s.lower()   # ID: callone
+                u = s.lower()   # ID: calltwo
+                lst.append(t)
+                lst.append(u)
+            return len(','.join(lst))
+        log = self.run(main, [1000])
+        assert log.result == main(1000)
+        loops = log.loops_by_filename(self.filepath)
+        loop, = loops
+        assert loop.match_by_id('callone', '''
+            i136 = strlen(p131)
+            i137 = int_eq(i135, i136)
+            guard_false(i137, descr=...)
+            p139 = call_r(ConstClass(_lower_unicode), p131, descr=...)
+            guard_no_exception(descr=...)
+            guard_nonnull(p139, descr=...)
+            ''')
+        assert loop.match_by_id('calltwo', '')    # nothing
+
     def test_move_method_call_out_of_loop(self):
         def main(n):
             lst = []
@@ -209,9 +238,82 @@ class TestString(BaseTestPyPyC):
         guard_not_invalidated(descr=...)
         p80 = call_r(ConstClass(ll_str__IntegerR_SignedConst_Signed), i47, descr=<Callr . i EF=3>)
         guard_no_exception(descr=...)
-        p53 = call_r(ConstClass(fast_str_decode_ascii), p80, descr=<Callr . r EF=4>)
+        i51 = call_i(ConstClass(first_non_ascii_char), p80, descr=<Calli . r EF=4>)
         guard_no_exception(descr=...)
+        i52 = int_lt(i51, 0)
+        guard_true(i52, descr=...)
+        i53 = strlen(p80)
         --TICK--
         jump(..., descr=...)
         """)
-        # XXX remove the guard_nonnull above?
+
+    def test_unicode_indexing_makes_no_bridges(self):
+        log = self.run("""
+        b = b"b'aaaaa\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'"
+        u = b.decode("utf-8") * 1000
+        def main():
+            for j in range(10):
+                for i in range(len(u)):
+                    u[i] # ID: index0
+        """, [])
+        ops = log.loops[0].ops_by_id("index0")
+        for op in ops:
+            assert op.bridge is None
+
+    def test_unicode_indexing_small_constant_indices(self):
+        log = self.run("""
+        l = [u"abä", u"cdä", u"äü", u"éé", u"–—¿"] * 1000
+        def main(n):
+            global s
+            for u in l:
+                s = u[0] + u[1] + u[-1] # ID: index
+                len(u)
+            return len(s)
+        """, [1000])
+        loop, = log.loops_by_filename(self.filepath)
+        assert loop.match_by_id('index', '''
+            i77 = getfield_gc_i(p73, descr=<FieldS pypy.objspace.std.unicodeobject.W_UnicodeObject.inst__length .*>)
+            p78 = getfield_gc_r(p73, descr=<FieldP pypy.objspace.std.unicodeobject.W_UnicodeObject.inst__utf8 .* pure>)
+            i79 = strlen(p78)
+            i80 = int_eq(i77, i79)
+            guard_false(i80, descr=...) # check not ascii
+            i82 = int_ge(0, i77)
+            guard_false(i82, descr=...)
+            i85 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p78, 0, descr=...)
+            i86 = int_gt(i85, i79)
+            guard_false(i86, descr=...)
+            i88 = int_ge(1, i77)
+            guard_false(i88, descr=...)
+            i90 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p78, i85, descr=...)
+            i91 = int_gt(i90, i79)
+            guard_false(i91, descr=...)
+            i92 = int_sub(i90, i85)
+            i94 = int_add(-1, i77)
+            i96 = call_i(ConstClass(prev_codepoint_pos_dont_look_inside), p78, i79, descr=...)
+            i97 = int_sub(i79, i96)
+            guard_not_invalidated(descr=...)
+        ''')
+
+    def test_unicode_slicing_small_constant_indices(self):
+        log = self.run("""
+        def main(n):
+            b = b'ab\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'
+            u = b.decode("utf-8") * 1000
+            global s
+            count = 0
+            while u:
+                u = u[1:] # ID: index
+                count += 1
+            return count
+        """, [1000])
+        loop, = log.loops_by_filename(self.filepath)
+        assert loop.match_by_id('index', '''
+            i51 = int_eq(1, i38)
+            guard_false(i51, descr=...)
+            i52 = strlen(p47)
+            i53 = int_eq(i38, i52)
+            guard_false(i53, descr=...)
+            i56 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p47, 0, descr=...)
+            i57 = int_sub(i52, i56)
+            i59 = int_sub(i38, 1)
+        ''')
