@@ -128,7 +128,7 @@ class Handlers:
             if WIN32 and signum not in signal_values:
                 self.handlers_w[signum] = space.w_None
             else:
-                self.handlers_w[signum] = space.wrap(SIG_DFL)
+                self.handlers_w[signum] = space.newint(SIG_DFL)
 
 def _get_handlers(space):
     return space.fromcache(Handlers).handlers_w
@@ -146,8 +146,8 @@ def report_signal(space, n):
     pypysig_reinstall(n)
     # invoke the app-level handler
     ec = space.getexecutioncontext()
-    w_frame = space.wrap(ec.gettopframe_nohidden())
-    space.call_function(w_handler, space.wrap(n), w_frame)
+    w_frame = ec.gettopframe_nohidden()
+    space.call_function(w_handler, space.newint(n), w_frame)
 
 
 @unwrap_spec(signum=int)
@@ -166,13 +166,14 @@ def getsignal(space, signum):
     return handlers_w[signum]
 
 
-def default_int_handler(space, w_signum, w_frame):
+def default_int_handler(space, args_w):
     """
     default_int_handler(...)
 
     The default handler for SIGINT installed by Python.
     It raises KeyboardInterrupt.
     """
+    # issue #2780: accept and ignore any non-keyword arguments
     raise OperationError(space.w_KeyboardInterrupt, space.w_None)
 
 
@@ -183,7 +184,7 @@ def alarm(space, timeout):
 
     Arrange for SIGALRM to arrive after the given number of seconds.
     """
-    return space.wrap(c_alarm(timeout))
+    return space.newint(c_alarm(timeout))
 
 
 @jit.dont_look_inside
@@ -224,9 +225,9 @@ def signal(space, signum, w_handler):
                     "__pypy__.thread.enable_signals()")
     check_signum_in_range(space, signum)
 
-    if space.eq_w(w_handler, space.wrap(SIG_DFL)):
+    if space.eq_w(w_handler, space.newint(SIG_DFL)):
         pypysig_default(signum)
-    elif space.eq_w(w_handler, space.wrap(SIG_IGN)):
+    elif space.eq_w(w_handler, space.newint(SIG_IGN)):
         pypysig_ignore(signum)
     else:
         if not space.is_true(space.callable(w_handler)):
@@ -260,7 +261,7 @@ def set_wakeup_fd(space, fd):
             if e.errno == errno.EBADF:
                 raise oefmt(space.w_ValueError, "invalid fd")
     old_fd = pypysig_set_wakeup_fd(fd, True)
-    return space.wrap(intmask(old_fd))
+    return space.newint(intmask(old_fd))
 
 
 @jit.dont_look_inside
@@ -275,14 +276,19 @@ def siginterrupt(space, signum, flag):
     check_signum_in_range(space, signum)
     if rffi.cast(lltype.Signed, c_siginterrupt(signum, flag)) < 0:
         errno = rposix.get_saved_errno()
-        raise OperationError(space.w_RuntimeError, space.wrap(errno))
+        raise OperationError(space.w_RuntimeError, space.newint(errno))
 
 
 #__________________________________________________________
 
 def timeval_from_double(d, timeval):
-    rffi.setintfield(timeval, 'c_tv_sec', int(d))
-    rffi.setintfield(timeval, 'c_tv_usec', int((d - int(d)) * 1000000))
+    c_tv_sec = int(d)
+    c_tv_usec = int((d - int(d)) * 1000000)
+    # Don't disable the timer if the computation above rounds down to zero.
+    if d > 0.0 and c_tv_sec == 0 and c_tv_usec == 0:
+        c_tv_usec = 1
+    rffi.setintfield(timeval, 'c_tv_sec', c_tv_sec)
+    rffi.setintfield(timeval, 'c_tv_usec', c_tv_usec)
 
 
 def double_from_timeval(tv):
@@ -291,8 +297,8 @@ def double_from_timeval(tv):
 
 
 def itimer_retval(space, val):
-    w_value = space.wrap(double_from_timeval(val.c_it_value))
-    w_interval = space.wrap(double_from_timeval(val.c_it_interval))
+    w_value = space.newfloat(double_from_timeval(val.c_it_value))
+    w_interval = space.newfloat(double_from_timeval(val.c_it_interval))
     return space.newtuple([w_value, w_interval])
 
 
