@@ -30,6 +30,7 @@ class TestCheckSignals:
         assert not space.is_true(w_received)
         #
         # calling ec.checksignals() should call it
+        print(space.getexecutioncontext().checksignals)
         space.getexecutioncontext().checksignals()
         assert space.is_true(w_received)
 
@@ -260,6 +261,17 @@ class AppTestSignal:
         finally:
             signal.signal(signum, oldhandler)
 
+    def test_default_int_handler(self):
+        import signal
+        for args in [(), (1, 2)]:
+            try:
+                signal.default_int_handler(*args)
+            except KeyboardInterrupt:
+                pass
+            else:
+                raise AssertionError("did not raise!")
+
+
 class AppTestSignalSocket:
     spaceconfig = dict(usemodules=['signal', '_socket'])
 
