@@ -1,5 +1,4 @@
 # -*- coding: iso-8859-1 -*-
-import codecs
 import sys
 
 def test_stdin_exists(space):
@@ -10,54 +9,11 @@ def test_stdout_exists(space):
     space.sys.get('stdout')
     space.sys.get('__stdout__')
 
-def test_stdout_flush_at_shutdown(space):
-    w_sys = space.sys
-    w_read = space.appexec([], """():
-        import sys
-        from io import BytesIO, TextIOWrapper
-        class BadWrite(BytesIO):
-            def write(self, data):
-                raise IOError
-        buf = BytesIO()
-        def read():
-            buf.seek(0)
-            return buf.read()
-        sys.stdout = TextIOWrapper(BadWrite())
-        sys.stderr = TextIOWrapper(buf)
-        return read""")
-
-    try:
-        space.call_method(w_sys.get('stdout'), 'write', space.wrap('x'))
-        # called at shtudown
-        w_sys.flush_std_files(space)
-
-        msg = space.bytes_w(space.call_function(w_read))
-        assert 'Exception ignored in:' in msg and '\nOSError' in msg
-    finally:
-        space.setattr(w_sys, space.wrap('stdout'), w_sys.get('__stdout__'))
-        space.setattr(w_sys, space.wrap('stderr'), w_sys.get('__stderr__'))
-
-def test_stdio_missing_at_shutdown(space):
-    w_sys = space.sys
-
-    try:
-        space.delattr(w_sys, space.wrap('stdout'))
-        space.delattr(w_sys, space.wrap('stderr'))
-        # called at shtudown
-        w_sys.flush_std_files(space)
-    finally:
-        space.setattr(w_sys, space.wrap('stdout'), w_sys.get('__stdout__'))
-        space.setattr(w_sys, space.wrap('stderr'), w_sys.get('__stderr__'))
-
 class AppTestAppSysTests:
-    spaceconfig = {
-        "usemodules": ["thread"],
-    }
 
     def setup_class(cls):
         cls.w_appdirect = cls.space.wrap(cls.runappdirect)
-        filesystemenc = codecs.lookup(sys.getfilesystemencoding()).name
-        cls.w_filesystemenc = cls.space.wrap(filesystemenc)
+        cls.w_filesystemenc = cls.space.wrap(sys.getfilesystemencoding())
 
     def test_sys_in_modules(self):
         import sys
@@ -69,40 +25,55 @@ class AppTestAppSysTests:
     def test_builtin_in_modules(self):
         import sys
         modules = sys.modules
-        assert 'builtins' in modules, ( "An entry for builtins "
-                                       "is not in sys.modules.")
-        import builtins
-        builtin2 = sys.modules['builtins']
-        assert builtins is builtin2, ( "import builtins "
-                                       "is not sys.modules[builtins].")
-
+        assert '__builtin__' in modules, ( "An entry for __builtin__ "
+                                                    "is not in sys.modules.")
+        import __builtin__
+        builtin2 = sys.modules['__builtin__']
+        assert __builtin__ is builtin2, ( "import __builtin__ "
+                                            "is not sys.modules[__builtin__].")
     def test_builtin_module_names(self):
         import sys
         names = sys.builtin_module_names
         assert 'sys' in names, (
-            "sys is not listed as a builtin module.")
-        assert 'builtins' in names, (
-            "builtins is not listed as a builtin module.")
-        assert 'exceptions' not in names, (
-            "exceptions module shouldn't exist")
+                        "sys is not listed as a builtin module.")
+        assert '__builtin__' in names, (
+                        "__builtin__ is not listed as a builtin module.")
 
     def test_sys_exc_info(self):
         try:
             raise Exception
-        except Exception as exc:
-            e = exc
+        except Exception as e:
             import sys
             exc_type,exc_val,tb = sys.exc_info()
         try:
-            raise Exception   # 6 lines below the previous one
-        except Exception as exc:
-            e2 = exc
+            raise Exception   # 5 lines below the previous one
+        except Exception as e2:
             exc_type2,exc_val2,tb2 = sys.exc_info()
         assert exc_type ==Exception
         assert exc_val ==e
         assert exc_type2 ==Exception
         assert exc_val2 ==e2
-        assert tb2.tb_lineno - tb.tb_lineno == 6
+        assert tb2.tb_lineno - tb.tb_lineno == 5
+
+    def test_dynamic_attributes(self):
+        try:
+            raise Exception
+        except Exception as e:
+            import sys
+            exc_type = sys.exc_type
+            exc_val = sys.exc_value
+            tb = sys.exc_traceback
+        try:
+            raise Exception   # 7 lines below the previous one
+        except Exception as e2:
+            exc_type2 = sys.exc_type
+            exc_val2 = sys.exc_value
+            tb2 = sys.exc_traceback
+        assert exc_type ==Exception
+        assert exc_val ==e
+        assert exc_type2 ==Exception
+        assert exc_val2 ==e2
+        assert tb2.tb_lineno - tb.tb_lineno == 7
 
     def test_exc_info_normalization(self):
         import sys
@@ -115,33 +86,25 @@ class AppTestAppSysTests:
             raise AssertionError("ZeroDivisionError not caught")
 
     def test_io(self):
-        import sys, io
-        assert isinstance(sys.__stdout__, io.IOBase)
-        assert isinstance(sys.__stderr__, io.IOBase)
-        assert isinstance(sys.__stdin__, io.IOBase)
-        assert sys.__stderr__.errors == 'backslashreplace'
+        import sys
+        assert isinstance(sys.__stdout__, file)
+        assert isinstance(sys.__stderr__, file)
+        assert isinstance(sys.__stdin__, file)
 
         #assert sys.__stdin__.name == "<stdin>"
         #assert sys.__stdout__.name == "<stdout>"
         #assert sys.__stderr__.name == "<stderr>"
 
-        if self.appdirect and not isinstance(sys.stdin, io.IOBase):
+        if self.appdirect and not isinstance(sys.stdin, file):
             return
 
-        assert isinstance(sys.stdout, io.IOBase)
-        assert isinstance(sys.stderr, io.IOBase)
-        assert isinstance(sys.stdin, io.IOBase)
-        assert sys.stderr.errors == 'backslashreplace'
+        assert isinstance(sys.stdout, file)
+        assert isinstance(sys.stderr, file)
+        assert isinstance(sys.stdin, file)
 
     def test_getfilesystemencoding(self):
         import sys
-        enc = sys.getfilesystemencoding()
-        # even before bootstraping, the encoding should match
-        assert enc == self.filesystemenc
-        if not self.appdirect:
-            # see comment in 'setup_after_space_initialization'
-            untranslated_enc = {'win32': 'utf-8', 'darwin': 'utf-8'}.get(enc, 'utf-8')
-            assert enc == untranslated_enc
+        assert sys.getfilesystemencoding() == self.filesystemenc
 
     def test_float_info(self):
         import sys
@@ -158,9 +121,9 @@ class AppTestAppSysTests:
         assert isinstance(fi.radix, int)
         assert isinstance(fi.rounds, int)
 
-    def test_int_info(self):
+    def test_long_info(self):
         import sys
-        li = sys.int_info
+        li = sys.long_info
         assert isinstance(li.bits_per_digit, int)
         assert isinstance(li.sizeof_digit, int)
 
@@ -178,105 +141,45 @@ class AppTestAppSysTests:
         exc = raises(SystemExit, sys.exit, (1, 2, 3))
         assert exc.value.code == (1, 2, 3)
 
-    def test_hash_info(self):
-        import sys
-        li = sys.hash_info
-        assert isinstance(li.width, int)
-        assert isinstance(li.modulus, int)
-        assert isinstance(li.inf, int)
-        assert isinstance(li.nan, int)
-        assert isinstance(li.imag, int)
-        assert isinstance(li.algorithm, str)
-
     def test_sys_flags(self):
         import sys
         # sanity check
         assert sys.flags.optimize is not None
         # make sure the flags are read-only
-        exc = raises(AttributeError, 'sys.flags.optimize = 3')
+        exc = raises(TypeError, 'sys.flags.optimize = 3')
         assert 'readonly' in str(exc.value)
         raises(AttributeError, 'sys.flags.not_a_sys_flag = 2')
 
-    def test_sys_exit(self):
-        import sys
-        exc = raises(SystemExit, sys.exit)
-        assert exc.value.code is None
 
-        exc = raises(SystemExit, sys.exit, 0)
-        assert exc.value.code == 0
-
-        exc = raises(SystemExit, sys.exit, 1)
-        assert exc.value.code == 1
-
-        exc = raises(SystemExit, sys.exit, (1, 2, 3))
-        assert exc.value.code == (1, 2, 3)
-
-    def test_sys_thread_info(self):
-        import sys
-        info = sys.thread_info
-        assert isinstance(info.name, str)
-        assert isinstance(info.lock, (str, type(None)))
-        assert isinstance(info.version, (str, type(None)))
-
-    def test_sys_flags_dev_mode_is_bool(self):
-        import sys
-        assert type(sys.flags.dev_mode) is bool
-
-    def test_multiarch(self):
-        import sys
-        if sys.platform == 'linux':
-            multiarch = sys.implementation._multiarch
-            assert 'linux' in multiarch
-        else:
-            assert not sys.implemenation.hasattr('_multiarch')
-
-    def test_audit(self):
-        import sys
-        sys.audit("os.chdir", "bla", 1, 2, 12) # does not crash
 class AppTestSysModulePortedFromCPython:
-    spaceconfig = {
-        "usemodules": ["struct"],
-    }
 
     def setup_class(cls):
         cls.w_appdirect = cls.space.wrap(cls.runappdirect)
 
     def test_original_displayhook(self):
-        import sys, _io, builtins
+        import sys, cStringIO, __builtin__
         savestdout = sys.stdout
-        out = _io.StringIO()
+        out = cStringIO.StringIO()
         sys.stdout = out
 
         dh = sys.__displayhook__
 
         raises(TypeError, dh)
-        if hasattr(builtins, "_"):
-            del builtins._
+        if hasattr(__builtin__, "_"):
+            del __builtin__._
 
         dh(None)
         assert out.getvalue() == ""
-        assert not hasattr(builtins, "_")
+        assert not hasattr(__builtin__, "_")
         dh("hello")
         assert out.getvalue() == "'hello'\n"
-        assert builtins._ == "hello"
+        assert __builtin__._ == "hello"
 
         del sys.stdout
         raises(RuntimeError, dh, 42)
 
         sys.stdout = savestdout
 
-    def test_original_displayhook_unencodable(self):
-        import sys, _io
-        out = _io.BytesIO()
-        savestdout = sys.stdout
-        sys.stdout = _io.TextIOWrapper(out, encoding='ascii')
-
-        sys.__displayhook__("a=\xe9 b=\uDC80 c=\U00010000 d=\U0010FFFF")
-        assert (out.getvalue() ==
-                b"'a=\\xe9 b=\\udc80 c=\\U00010000 d=\\U0010ffff'")
-
-        sys.stdout = savestdout
-
     def test_lost_displayhook(self):
         import sys
         olddisplayhook = sys.displayhook
@@ -296,9 +199,9 @@ class AppTestSysModulePortedFromCPython:
         sys.displayhook = olddisplayhook
 
     def test_original_excepthook(self):
-        import sys, _io
+        import sys, cStringIO
         savestderr = sys.stderr
-        err = _io.StringIO()
+        err = cStringIO.StringIO()
         sys.stderr = err
 
         eh = sys.__excepthook__
@@ -308,21 +211,16 @@ class AppTestSysModulePortedFromCPython:
             raise ValueError(42)
         except ValueError as exc:
             eh(*sys.exc_info())
-        assert err.getvalue().endswith("ValueError: 42\n")
-
-        eh(1, '1', 1)
-        expected = ("TypeError: print_exception(): Exception expected for "
-                    "value, str found")
-        assert expected in err.getvalue()
 
         sys.stderr = savestderr
+        assert err.getvalue().endswith("ValueError: 42\n")
 
     def test_excepthook_failsafe_path(self):
         import traceback
         original_print_exception = traceback.print_exception
-        import sys, _io
+        import sys, cStringIO
         savestderr = sys.stderr
-        err = _io.StringIO()
+        err = cStringIO.StringIO()
         sys.stderr = err
         try:
             traceback.print_exception = "foo"
@@ -351,7 +249,8 @@ class AppTestSysModulePortedFromCPython:
             def getvalue(self):
                 return ''.join(self.output)
 
-        for input in ("\u013a", "\u1111"):
+        for input, expectedoutput in [(u"\u013a", "\xe5"),
+                                      (u"\u1111", "\\u1111")]:
             err = MyStringIO()
             err.encoding = 'iso-8859-2'
             sys.stderr = err
@@ -363,13 +262,13 @@ class AppTestSysModulePortedFromCPython:
                 eh(*sys.exc_info())
 
             sys.stderr = savestderr
-            print(ascii(err.getvalue()))
-            assert err.getvalue().endswith("ValueError: %s\n" % input)
+            print repr(err.getvalue())
+            assert err.getvalue().endswith("ValueError: %s\n" % expectedoutput)
 
-    def test_excepthook_flushes_stdout(self): r"""
-        import sys, io
+    def test_excepthook_flushes_stdout(self):
+        import sys, cStringIO
         savestdout = sys.stdout
-        out = io.StringIO()
+        out = cStringIO.StringIO()
         sys.stdout = out
 
         eh = sys.__excepthook__
@@ -377,62 +276,67 @@ class AppTestSysModulePortedFromCPython:
         try:
             raise ValueError(42)
         except ValueError as exc:
-            print("hello")     # with end-of-line
+            print "hello"     # with end-of-line
             eh(*sys.exc_info())
         try:
             raise ValueError(42)
         except ValueError as exc:
-            print(123, 456, end="")     # no end-of-line here
+            print 123, 456,     # no end-of-line here
+            assert sys.stdout.softspace
             eh(*sys.exc_info())
+            assert not sys.stdout.softspace
 
         sys.stdout = savestdout
-        assert out.getvalue() == 'hello\n123 456'   # no final \n added in 3.x
-        """
-
-    def test_tracebacklimit_excepthook(self):
-        import sys
-        savestderr = sys.stderr
-        assert not hasattr(sys, "tracebacklimit")
-
-        eh = sys.__excepthook__
-        def f1():
-            f2()
-        def f2():
-            f3()
-        def f3():
-            raise ValueError(42)
-
-        def get_error_with_tracebacklimit(limit):
-            import io
-            sys.tracebacklimit = limit
-            sys.stderr = err = io.StringIO()
-            try:
-                f1()
-            except ValueError:
-                eh(*sys.exc_info())
-            return err.getvalue()
-            # should be removed by the limit
-
-        msg = get_error_with_tracebacklimit(2)
-        assert "f1" not in msg
-        assert "f2" in msg
-        assert "f3" in msg
-
-        msg = get_error_with_tracebacklimit(0)
-        assert "Traceback (most recent call last):" not in msg
-        assert "ValueError" in msg
-
-        msg = get_error_with_tracebacklimit(-1)
-        assert "Traceback (most recent call last):" not in msg
-        assert "ValueError" in msg
-
-        sys.stderr = savestderr
-        del sys.tracebacklimit
-
+        assert out.getvalue() == 'hello\n123 456\n'   # with a final \n added
 
     # FIXME: testing the code for a lost or replaced excepthook in
     # Python/pythonrun.c::PyErr_PrintEx() is tricky.
 
+    def test_exc_clear(self):
+        import sys
+        raises(TypeError, sys.exc_clear, 42)
+
+        # Verify that exc_info is present and matches exc, then clear it, and
+        # check that it worked.
+        def clear_check(exc):
+            typ, value, traceback = sys.exc_info()
+            assert typ is not None
+            assert value is exc
+            assert traceback is not None
+
+            sys.exc_clear()
+
+            typ, value, traceback = sys.exc_info()
+            assert typ is None
+            assert value is None
+            assert traceback is None
+
+        def clear():
+            try:
+                raise ValueError(42)
+            except ValueError as exc:
+                clear_check(exc)
+
+        # Raise an exception and check that it can be cleared
+        clear()
+
+        # Verify that a frame currently handling an exception is
+        # unaffected by calling exc_clear in a nested frame.
+        try:
+            raise ValueError(13)
+        except ValueError as exc:
+            typ1, value1, traceback1 = sys.exc_info()
+            clear()
+            typ2, value2, traceback2 = sys.exc_info()
+
+            assert typ1 is typ2
+            assert value1 is exc
+            assert value1 is value2
+            assert traceback1 is traceback2
+
+        # Check that an exception can be cleared outside of an except block
+        clear_check(exc)
+
     def test_exit(self):
         import sys
         raises(TypeError, sys.exit, 42, 42)
@@ -494,6 +398,25 @@ class AppTestSysModulePortedFromCPython:
         # can't check more than the type, as the user might have changed it
         assert isinstance(sys.getdefaultencoding(), str)
 
+    def test_setdefaultencoding(self):
+        import sys
+        if self.appdirect:
+            skip("not worth running appdirect")
+
+        encoding = sys.getdefaultencoding()
+        try:
+            sys.setdefaultencoding("ascii")
+            assert sys.getdefaultencoding() == 'ascii'
+            raises(UnicodeDecodeError, unicode, '\x80')
+
+            sys.setdefaultencoding("latin-1")
+            assert sys.getdefaultencoding() == 'latin-1'
+            assert unicode('\x80') == u'\u0080'
+
+        finally:
+            sys.setdefaultencoding(encoding)
+
+
     # testing sys.settrace() is done in test_trace.py
     # testing sys.setprofile() is done in test_profile.py
 
@@ -505,15 +428,6 @@ class AppTestSysModulePortedFromCPython:
             sys.setcheckinterval(n)
             assert sys.getcheckinterval() == n
 
-    def test_setswitchinterval(self):
-        import sys
-        raises(TypeError, sys.setswitchinterval)
-        orig = sys.getswitchinterval()
-        for n in 1e-6, 0.1, orig: # orig last to restore starting state
-            sys.setswitchinterval(n)
-            assert sys.getswitchinterval() == n
-        raises(ValueError, sys.setswitchinterval, 0.0)
-
     def test_recursionlimit(self):
         import sys
         raises(TypeError, sys.getrecursionlimit, 42)
@@ -525,16 +439,6 @@ class AppTestSysModulePortedFromCPython:
         sys.setrecursionlimit(oldlimit)
         raises(OverflowError, sys.setrecursionlimit, 1<<31)
 
-    def test_recursionlimit_toolow(self):
-        import sys
-        def callatlevel(l):
-            if l > 0:
-                callatlevel(l - 1)
-            else:
-                sys.setrecursionlimit(1)
-        with raises(RecursionError):
-            callatlevel(500)
-
     def test_getwindowsversion(self):
         import sys
         if hasattr(sys, "getwindowsversion"):
@@ -627,15 +531,16 @@ class AppTestSysModulePortedFromCPython:
         assert isinstance(sys.argv, list)
         assert sys.byteorder in ("little", "big")
         assert isinstance(sys.builtin_module_names, tuple)
-        assert isinstance(sys.copyright, str)
-        #assert isinstance(sys.exec_prefix, str) -- not present!
-        #assert isinstance(sys.executable, str)
+        assert isinstance(sys.copyright, basestring)
+        #assert isinstance(sys.exec_prefix, basestring) -- not present!
+        #assert isinstance(sys.executable, basestring) -- not present!
         assert isinstance(sys.hexversion, int)
+        assert isinstance(sys.maxint, int)
         assert isinstance(sys.maxsize, int)
         assert isinstance(sys.maxunicode, int)
-        assert isinstance(sys.platform, str)
-        #assert isinstance(sys.prefix, str) -- not present!
-        assert isinstance(sys.version, str)
+        assert isinstance(sys.platform, basestring)
+        #assert isinstance(sys.prefix, basestring) -- not present!
+        assert isinstance(sys.version, basestring)
         assert isinstance(sys.warnoptions, list)
         vi = sys.version_info
         if '__pypy__' in sys.builtin_module_names:
@@ -646,83 +551,9 @@ class AppTestSysModulePortedFromCPython:
         assert isinstance(vi[2], int)
         assert vi[3] in ("alpha", "beta", "candidate", "final")
         assert isinstance(vi[4], int)
-        assert isinstance(sys._framework, str)
-
-    def test_implementation(self):
-        import sys
-        assert sys.implementation.name == 'pypy'
-
-        # This test applies to all implementations equally.
-        levels = {'alpha': 0xA, 'beta': 0xB, 'candidate': 0xC, 'final': 0xF}
-
-        assert sys.implementation.version
-        assert sys.implementation.hexversion
-        assert sys.implementation.cache_tag
-
-        version = sys.implementation.version
-        assert version[:2] == (version.major, version.minor)
-
-        hexversion = (version.major << 24 | version.minor << 16 |
-                      version.micro << 8 | levels[version.releaselevel] << 4 |
-                      version.serial << 0)
-        assert sys.implementation.hexversion == hexversion
-
-        # PEP 421 requires that .name be lower case.
-        assert sys.implementation.name == sys.implementation.name.lower()
-
-        ns1 = type(sys.implementation)(x=1, y=2, w=3)
-        assert repr(ns1) == "namespace(w=3, x=1, y=2)"
-
-    def test_simplenamespace(self):
-        import sys
-        SimpleNamespace = type(sys.implementation)
-        ns = SimpleNamespace(x=1, y=2, w=3)
-        #
-        ns.z = 4
-        assert ns.__dict__ == dict(x=1, y=2, w=3, z=4)
-        #
-        raises(AttributeError, "del ns.spam")
-        del ns.y
-        #
-        assert ns == SimpleNamespace(z=4, x=1, w=3)
-        assert (ns != SimpleNamespace(z=4, x=1, w=3)) is False
-        assert (ns == SimpleNamespace(z=4, x=2, w=3)) is False
-        assert ns != SimpleNamespace(z=4, x=2, w=3)
-        #
-        class Foo(SimpleNamespace):
-            pass
-        assert ns == Foo(z=4, x=1, w=3)
-        assert (ns != Foo(z=4, x=1, w=3)) is False
-        assert (ns == Foo(z=4, x=2, w=3)) is False
-        assert ns != Foo(z=4, x=2, w=3)
-        #
-        class Other:
-            def __init__(self, x, z, w):
-                self.x = x
-                self.z = z
-                self.w = w
-        assert (ns == Other(z=4, x=1, w=3)) is False
-        assert ns != Other(z=4, x=1, w=3)
-        assert (Foo(z=4, x=1, w=3) == Other(z=4, x=1, w=3)) is False
-        assert Foo(z=4, x=1, w=3) != Other(z=4, x=1, w=3)
-        #
-        class Fake:
-            __class__ = SimpleNamespace
-        assert isinstance(Fake(), SimpleNamespace)
-        assert not issubclass(Fake, SimpleNamespace)
-        assert (Fake() == SimpleNamespace()) is False
-        assert SimpleNamespace() != Fake()
-
-    def test_pickle_simplenamespace(self):
-        import pickle, sys
-        SimpleNamespace = type(sys.implementation)
-        ns = SimpleNamespace(x=1, y=2, w=3)
-        copy = pickle.loads(pickle.dumps(ns))
-        assert copy == ns
 
     def test_reload_doesnt_override_sys_executable(self):
         import sys
-        from imp import reload
         if not hasattr(sys, 'executable'):    # if not translated
             sys.executable = 'from_test_sysmodule'
         previous = sys.executable
@@ -784,6 +615,24 @@ class AppTestSysModulePortedFromCPython:
         if hgid != '':
             assert hgid in sys.version
 
+    def test_trace_exec_execfile(self):
+        import sys
+        found = []
+        def do_tracing(f, *args):
+            print f.f_code.co_filename, f.f_lineno, args
+            if f.f_code.co_filename == 'foobar':
+                found.append(args[0])
+            return do_tracing
+        co = compile("execfile('this-file-does-not-exist!')",
+                     'foobar', 'exec')
+        sys.settrace(do_tracing)
+        try:
+            exec co in {}
+        except IOError:
+            pass
+        sys.settrace(None)
+        assert found == ['call', 'line', 'exception', 'return']
+
     def test_float_repr_style(self):
         import sys
 
@@ -791,51 +640,6 @@ class AppTestSysModulePortedFromCPython:
         # be changed.
         assert sys.float_repr_style == "short"
 
-    def test_is_finalizing(self):
-        import sys
-        assert not sys.is_finalizing()
-        # xxx should also test when it is True, but maybe not worth the effort
-
-    def test_asyncgen_hooks(self):
-        import sys
-        old = sys.get_asyncgen_hooks()
-        assert old.firstiter is None
-        assert old.finalizer is None
-
-        firstiter = lambda *a: None
-        sys.set_asyncgen_hooks(firstiter=firstiter)
-        hooks = sys.get_asyncgen_hooks()
-        assert hooks.firstiter is firstiter
-        assert hooks[0] is firstiter
-        assert hooks.finalizer is None
-        assert hooks[1] is None
-
-        finalizer = lambda *a: None
-        sys.set_asyncgen_hooks(finalizer=finalizer)
-        hooks = sys.get_asyncgen_hooks()
-        assert hooks.firstiter is firstiter
-        assert hooks[0] is firstiter
-        assert hooks.finalizer is finalizer
-        assert hooks[1] is finalizer
-
-        sys.set_asyncgen_hooks(*old)
-        cur = sys.get_asyncgen_hooks()
-        assert cur.firstiter is None
-        assert cur.finalizer is None
-
-    def test_coroutine_origin_tracking_depth(self):
-        import sys
-        depth = sys.get_coroutine_origin_tracking_depth()
-        assert depth == 0
-        try:
-            sys.set_coroutine_origin_tracking_depth(6)
-            depth = sys.get_coroutine_origin_tracking_depth()
-            assert depth == 6
-            with raises(ValueError):
-                sys.set_coroutine_origin_tracking_depth(-5)
-        finally:
-            sys.set_coroutine_origin_tracking_depth(0)
-
 class AppTestSysSettracePortedFromCpython(object):
     def test_sys_settrace(self):
         import sys
@@ -859,7 +663,7 @@ class AppTestSysSettracePortedFromCpython(object):
             tracer = Tracer()
             func(tracer.trace)
             sys.settrace(None)
-            compare_events(func.__code__.co_firstlineno,
+            compare_events(func.func_code.co_firstlineno,
                            tracer.events, func.events)
 
 
@@ -893,7 +697,7 @@ class AppTestSysSettracePortedFromCpython(object):
 class AppTestCurrentFrames:
     def test_current_frames(self):
         try:
-            import _thread
+            import thread
         except ImportError:
             pass
         else:
@@ -903,7 +707,7 @@ class AppTestCurrentFrames:
         def f():
             return sys._current_frames()
         frames = f()
-        assert list(frames) == [0]
+        assert frames.keys() == [0]
         assert frames[0].f_code.co_name in ('f', '?')
 
 
@@ -915,23 +719,23 @@ class AppTestCurrentFramesWithThread(AppTestCurrentFrames):
     def test_current_frames(self):
         import sys
         import time
-        import _thread
+        import thread
 
         # XXX workaround for now: to prevent deadlocks, call
         # sys._current_frames() once before starting threads.
         # This is an issue in non-translated versions only.
         sys._current_frames()
 
-        thread_id = _thread.get_ident()
+        thread_id = thread.get_ident()
         def other_thread():
-            #print("thread started")
+            #print "thread started"
             lock2.release()
             lock1.acquire()
-        lock1 = _thread.allocate_lock()
-        lock2 = _thread.allocate_lock()
+        lock1 = thread.allocate_lock()
+        lock2 = thread.allocate_lock()
         lock1.acquire()
         lock2.acquire()
-        _thread.start_new_thread(other_thread, ())
+        thread.start_new_thread(other_thread, ())
 
         def f():
             lock2.acquire()
@@ -946,24 +750,6 @@ class AppTestCurrentFramesWithThread(AppTestCurrentFrames):
         _, other_frame = frames.popitem()
         assert other_frame.f_code.co_name in ('other_thread', '?')
 
-    def test_intern(self):
-        from sys import intern
-        raises(TypeError, intern)
-        raises(TypeError, intern, 1)
-        class S(str):
-            pass
-        raises(TypeError, intern, S("hello"))
-        s = "never interned before"
-        s2 = intern(s)
-        assert s == s2
-        s3 = s.swapcase()
-        assert s3 != s2
-        s4 = s3.swapcase()
-        assert intern(s4) is s2
-        s5 = "\ud800"
-        # previously failed
-        assert intern(s5) == s5
-
 
 class AppTestSysExcInfoDirect:
 
@@ -1070,15 +856,12 @@ class AppTestSysExcInfoDirect:
     def test_call_in_subfunction(self):
         import sys
         def g():
-            # new in PyPy3: this case is optimized as well, now that we
-            # don't need to search the frame chain.  This can be
-            # regarded as an advantage of the PyPy3 sys_exc_info stored
-            # on the ExecutionContext: although it forces the exception
-            # itself more often, it forces frames less often.
+            # this case is not optimized, because we need to search the
+            # frame chain.  it's probably not worth the complications
             return sys.exc_info()[1]
         e = KeyError("boom")
         try:
             raise e
         except:
             assert g() is e
-    test_call_in_subfunction.expected = 'y'
+    test_call_in_subfunction.expected = 'n'
