@@ -14,7 +14,6 @@ class WRefShrinkList(AbstractShrinkList):
 
 
 ExecutionContext._thread_local_objs = None
-ExecutionContext._sentinel_lock = None
 
 
 class Local(W_Root):
@@ -78,7 +77,7 @@ class Local(W_Root):
     def descr_local__new__(space, w_subtype, __args__):
         from pypy.objspace.std.typeobject import _precheck_for_new
         w_subtype = _precheck_for_new(space, w_subtype)
-        if __args__.arguments_w or __args__.keyword_names_w:
+        if __args__.arguments_w or __args__.keywords:
             w_parent_init, _ = space.lookup_in_type_where(w_subtype, '__init__')
             if w_parent_init is space.w_object:
                 raise oefmt(space.w_TypeError,
@@ -87,7 +86,7 @@ class Local(W_Root):
         Local.__init__(local, space, __args__)
         return local
 
-Local.typedef = TypeDef("_thread._local",
+Local.typedef = TypeDef("thread._local",
                         __doc__ = "Thread-local data",
                         __new__ = interp2app(Local.descr_local__new__.im_func),
                         __dict__ = GetSetProperty(descr_get_dict, cls=Local),
@@ -95,10 +94,6 @@ Local.typedef = TypeDef("_thread._local",
                         )
 
 def thread_is_stopping(ec):
-    sentinel_lock = ec._sentinel_lock
-    if sentinel_lock is not None:
-        if sentinel_lock.lock.is_acquired():
-            sentinel_lock.descr_lock_release(ec.space)
     tlobjs = ec._thread_local_objs
     if tlobjs is None:
         return
