@@ -380,11 +380,19 @@ class AppTestBytesArray:
         assert b == bytearray('obe')
 
     def test_iadd(self):
-        b = bytearray('abc')
+        b = b0 = bytearray('abc')
         b += 'def'
         assert b == 'abcdef'
-        assert isinstance(b, bytearray)
+        assert b is b0
         raises(TypeError, b.__iadd__, u"")
+        #
+        b += bytearray('XX')
+        assert b == 'abcdefXX'
+        assert b is b0
+        #
+        b += memoryview('ABC')
+        assert b == 'abcdefXXABC'
+        assert b is b0
 
     def test_add(self):
         b1 = bytearray("abc")
@@ -448,6 +456,10 @@ class AppTestBytesArray:
         raises(TypeError, b.extend, [object()])
         raises(TypeError, b.extend, u"unicode")
 
+        b = bytearray('abc')
+        b.extend(memoryview('def'))
+        assert b == bytearray('abcdef')
+
     def test_extend_calls_len_or_lengthhint(self):
         class BadLen(object):
             def __iter__(self): return iter(range(10))
@@ -641,3 +653,15 @@ class AppTestBytesArray:
         assert x.find(b'fe') == -1
         assert x.index(b'f', 2, 11) == 5
         assert x.__alloc__() == 14
+
+    def test_fromobject___index__(self):
+        class WithIndex:
+            def __index__(self):
+                return 3
+        assert bytearray(WithIndex()) == b'\x00\x00\x00'
+
+    def test_fromobject___int__(self):
+        class WithInt:
+            def __int__(self):
+                return 3
+        raises(TypeError, bytearray, WithInt())
