@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 import sys
 import py
 
@@ -141,11 +142,12 @@ class TestW_DictObject(object):
         w_d.initialize_content([(wb("a"), w(1)), (wb("b"), w(2))])
         assert self.space.listview_bytes(w_d) == ["a", "b"]
 
+    @py.test.mark.skip("possible re-enable later?")
     def test_listview_unicode_dict(self):
         w = self.space.wrap
         w_d = self.space.newdict()
         w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(2))])
-        assert self.space.listview_unicode(w_d) == [u"a", u"b"]
+        assert self.space.listview_ascii(w_d) == ["a", "b"]
 
     def test_listview_int_dict(self):
         w = self.space.wrap
@@ -172,13 +174,14 @@ class TestW_DictObject(object):
         w_l = self.space.call_method(w_d, "keys")
         assert sorted(self.space.listview_bytes(w_l)) == ["a", "b"]
 
+        #---the rest is for listview_unicode(), which is disabled---
         # XXX: it would be nice if the test passed without monkeypatch.undo(),
         # but we need space.newlist_unicode for it
-        monkeypatch.undo()
-        w_d = self.space.newdict()
-        w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(6))])
-        w_l = self.space.call_method(w_d, "keys")
-        assert sorted(self.space.listview_unicode(w_l)) == [u"a", u"b"]
+        # monkeypatch.undo()
+        # w_d = self.space.newdict()
+        # w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(6))])
+        # w_l = self.space.call_method(w_d, "keys")
+        # assert sorted(self.space.listview_unicode(w_l)) == [u"a", u"b"]
 
 class AppTest_DictObject:
     def setup_class(cls):
@@ -338,7 +341,8 @@ class AppTest_DictObject:
             return k
         for last in [False, True]:
             for d, key in [({1: 2, 3: 4, 5: 6}, 3),
-                           ({"a": 5, "b": 2, "c": 6}, "b"),
+                           ({b"a": 5, b"b": 2, b"c": 6}, b"b"),
+                           ({u"a": 5, u"b": 2, u"c": 6}, u"b"),
                            (kwdict(d=7, e=8, f=9), "e")]:
                 other_keys = [k for k in d if k != key]
                 __pypy__.move_to_end(d, key, last=last)
@@ -913,6 +917,12 @@ class AppTestDictViews:
         assert (r == "dict_values(['ABC', 10])" or
                 r == "dict_values([10, 'ABC'])")
 
+    def test_recursive_repr(self):
+        d = {1: 2}
+        d[2] = d.viewvalues()
+        print repr(d)
+        assert repr(d) == '{1: 2, 2: dict_values([2, ...])}'
+
     def test_keys_set_operations(self):
         d1 = {'a': 1, 'b': 2}
         d2 = {'b': 3, 'c': 2}
@@ -1115,6 +1125,18 @@ class AppTestDictViews:
         assert (foo2, foo2_bis) in logger_copy
         assert logger_copy.issubset({(foo1, foo2_bis), (foo2, foo2_bis), (foo3, foo2_bis)})
 
+    def test_pickle(self):
+        d = {1: 1, 2: 2, 3: 3}
+        it = iter(d)
+        first = next(it)
+        reduced = it.__reduce__()
+        rebuild, args = reduced
+        new = rebuild(*args)
+        items = set(new)
+        assert len(items) == 2
+        items.add(first)
+        assert items == set(d)
+
 
 class AppTestStrategies(object):
     def setup_class(cls):
@@ -1150,8 +1172,11 @@ class AppTestStrategies(object):
         assert d.keys() == [u"a"]
         assert type(d.keys()[0]) is unicode
 
+        d = {}
+        d[u"ä"] = 1
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
+
     def test_empty_to_int(self):
-        import sys
         d = {}
         d[1] = "hi"
         assert "IntDictStrategy" in self.get_strategy(d)
@@ -1240,9 +1265,13 @@ class FakeSpace:
         assert isinstance(integer, int)
         return integer
 
+    def float_w(self, fl, allow_conversion=True):
+        assert isinstance(fl, float)
+        return fl
+
     def wrap(self, obj):
         return obj
-    newtext = newbytes = wrap
+    newtext = newbytes = newint = newfloat = wrap
 
     def isinstance_w(self, obj, klass):
         return isinstance(obj, klass)
@@ -1284,6 +1313,8 @@ class FakeSpace:
     w_float = float
     StringObjectCls = FakeString
     UnicodeObjectCls = FakeUnicode
+    IntObjectCls = int
+    FloatObjectCls = float
     w_dict = W_DictObject
     iter = iter
     fixedview = list
