@@ -377,6 +377,9 @@ class AppTestInt(object):
         assert a == 2
         a -= 1
         assert a == 1
+        e = raises(TypeError, "a += 'aa'")
+        assert "+=" in str(e.value)
+
 
     def test_trunc(self):
         import math
@@ -679,6 +682,20 @@ class AppTestInt(object):
         x = int(321)
         assert x.__rlshift__(333) == 1422567365923326114875084456308921708325401211889530744784729710809598337369906606315292749899759616L
 
+    def test_some_rops(self):
+        import sys
+        x = int(-sys.maxint)
+        assert x.__rsub__(2) == (2 + sys.maxint)
+
+    def test_error_message_wrong_self(self):
+        unboundmeth = int.__str__
+        e = raises(TypeError, unboundmeth, "!")
+        assert "int" in str(e.value)
+        if hasattr(unboundmeth, 'im_func'):
+            e = raises(TypeError, unboundmeth.im_func, "!")
+            assert "'int'" in str(e.value)
+
+
 class AppTestIntShortcut(AppTestInt):
     spaceconfig = {"objspace.std.intshortcut": True}
 
