@@ -2,7 +2,8 @@
 import py
 import random
 from pypy.objspace.std.listobject import W_ListObject, SizeListStrategy,\
-     IntegerListStrategy, ObjectListStrategy
+     IntegerListStrategy, BytesListStrategy, FloatListStrategy, \
+     ObjectListStrategy, IntOrFloatListStrategy, AsciiListStrategy
 from pypy.interpreter.error import OperationError
 from rpython.rlib.rarithmetic import is_valid_int
 
@@ -430,6 +431,66 @@ class TestW_ListObject(object):
         with py.test.raises(ValueError):
             intlist.find(w(4), 0, 2)
 
+    def test_intlist_to_object_too_much_wrapping(self):
+        w = self.space.wrap
+        w_list = W_ListObject(self.space, [w(1000000)] * 100)
+        assert isinstance(w_list.strategy, IntegerListStrategy)
+        w_list.setitem(0, w(1))
+        w_list.setitem(0, self.space.w_None)
+        assert isinstance(w_list.strategy, ObjectListStrategy)
+        l = w_list.getitems()
+        assert len(set(l)) == 2
+
+    def test_floatlist_to_object_too_much_wrapping(self):
+        w = self.space.wrap
+        for value in [11233.12, float('NaN')]:
+            w_list = W_ListObject(self.space, [w(value)] * 100)
+            assert isinstance(w_list.strategy, FloatListStrategy)
+            w_list.setitem(0, self.space.w_None)
+            assert isinstance(w_list.strategy, ObjectListStrategy)
+            l = w_list.getitems()
+            for element in l[2:]:
+                assert element is l[1]
+
+    def test_intorfloatlist_to_object_too_much_wrapping(self):
+        w = self.space.wrap
+        w_list = W_ListObject(self.space, [w(0)] * 100)
+        w_list.setitem(0, w(1.1))
+        assert isinstance(w_list.strategy, IntOrFloatListStrategy)
+        w_list.setitem(0, self.space.w_None)
+        assert isinstance(w_list.strategy, ObjectListStrategy)
+        l = w_list.getitems()
+        assert len(set(l)) == 2
+
+    def test_byteslist_to_object_too_much_wrapping(self):
+        w = self.space.wrap
+        w_list = W_ListObject(self.space, [self.space.newbytes(b"abc")] * 100)
+        assert isinstance(w_list.strategy, BytesListStrategy)
+        w_list.setitem(0, self.space.w_None)
+        assert isinstance(w_list.strategy, ObjectListStrategy)
+        l = w_list.getitems()
+        for element in l[2:]:
+            assert element is l[1]
+
+    def test_asciilist_to_object_too_much_wrapping(self):
+        w = self.space.wrap
+        w_list = W_ListObject(self.space, [self.space.newutf8(b"abc", 3)] * 100)
+        assert isinstance(w_list.strategy, AsciiListStrategy)
+        w_list.setitem(0, self.space.w_None)
+        assert isinstance(w_list.strategy, ObjectListStrategy)
+        l = w_list.getitems()
+        for element in l[2:]:
+            assert element is l[1]
+
+    def test_tuple_extend_shortcut(self, space, monkeypatch):
+        from pypy.objspace.std import listobject
+        w = self.space.wrap
+        w_list = W_ListObject(space, [w(5)])
+        w_tup = space.newtuple([w(6), w(7)])
+        monkeypatch.setattr(listobject, "_do_extend_from_iterable", None)
+        space.call_method(w_list, "extend", w_tup) # does not crash because of the shortcut
+        assert space.unwrap(w_list) == [5, 6, 7]
+
 
 class AppTestListObject(object):
     #spaceconfig = {"objspace.std.withliststrategies": True}  # it's the default
@@ -1130,6 +1191,24 @@ class AppTestListObject(object):
             b[i:i+1] = ['y']
         assert b == ['y'] * count
 
+    def test_setslice_full(self):
+        l = [1, 2, 3]
+        l[::] = "abc"
+        assert l == ['a', 'b', 'c']
+
+        l = [1, 2, 3]
+        l[::] = []
+        assert l == []
+
+        l = [1, 2, 3]
+        l[::] = l
+        assert l == [1, 2, 3]
+
+    def test_setslice_full_bug(self):
+        l = [1, 2, 3]
+        l[::] = (x + 1 for x in l)
+        assert l == [2, 3, 4]
+
     def test_recursive_repr(self):
         l = []
         assert repr(l) == '[]'
