@@ -80,10 +80,11 @@ class TestW_SetObject:
 
     def test_create_set_from_list(self):
         from pypy.interpreter.baseobjspace import W_Root
-        from pypy.objspace.std.setobject import BytesSetStrategy, ObjectSetStrategy, UnicodeSetStrategy
+        from pypy.objspace.std.setobject import BytesSetStrategy, ObjectSetStrategy
         from pypy.objspace.std.floatobject import W_FloatObject
 
         w = self.space.wrap
+        wb = self.space.newbytes
         intstr = self.space.fromcache(IntegerSetStrategy)
         tmp_func = intstr.get_storage_from_list
         # test if get_storage_from_list is no longer used
@@ -95,7 +96,7 @@ class TestW_SetObject:
         assert w_set.strategy is intstr
         assert intstr.unerase(w_set.sstorage) == {1:None, 2:None, 3:None}
 
-        w_list = W_ListObject(self.space, [w("1"), w("2"), w("3")])
+        w_list = W_ListObject(self.space, [wb("1"), wb("2"), wb("3")])
         w_set = W_SetObject(self.space)
         _initialize_set(self.space, w_set, w_list)
         assert w_set.strategy is self.space.fromcache(BytesSetStrategy)
@@ -104,8 +105,6 @@ class TestW_SetObject:
         w_list = self.space.iter(W_ListObject(self.space, [w(u"1"), w(u"2"), w(u"3")]))
         w_set = W_SetObject(self.space)
         _initialize_set(self.space, w_set, w_list)
-        assert w_set.strategy is self.space.fromcache(UnicodeSetStrategy)
-        assert w_set.strategy.unerase(w_set.sstorage) == {u"1":None, u"2":None, u"3":None}
 
         w_list = W_ListObject(self.space, [w("1"), w(2), w("3")])
         w_set = W_SetObject(self.space)
@@ -126,9 +125,10 @@ class TestW_SetObject:
 
     def test_listview_bytes_int_on_set(self):
         w = self.space.wrap
+        wb = self.space.newbytes
 
         w_a = W_SetObject(self.space)
-        _initialize_set(self.space, w_a, w("abcdefg"))
+        _initialize_set(self.space, w_a, wb("abcdefg"))
         assert sorted(self.space.listview_bytes(w_a)) == list("abcdefg")
         assert self.space.listview_int(w_a) is None
 
@@ -433,13 +433,18 @@ class AppTestAppSetTest:
         s1 = set([1, 2.0, "3"])
         s1.update(set(["3", 4, 5.0]))
 
+    def test_update_not_iterable_error(self):
+        with raises(TypeError) as e:
+            set().update(1)
+        assert "'int' object is not iterable" in str(e.value)
+
     def test_recursive_repr(self):
         class A(object):
             def __init__(self, s):
                 self.s = s
             def __repr__(self):
                 return repr(self.s)
-        
+
         s = set([1, 2, 3])
         s.add(A(s))
         therepr = repr(s)
@@ -460,12 +465,12 @@ class AppTestAppSetTest:
         assert therepr.endswith("])")
         inner = set(therepr[11:-2].split(", "))
         assert inner == set(["1", "2", "3", "frozenset(...)"])
-        
+
     def test_keyerror_has_key(self):
         s = set()
         try:
             s.remove(1)
-        except KeyError, e:
+        except KeyError as e:
             assert e.args[0] == 1
         else:
             assert 0, "should raise"
@@ -477,7 +482,7 @@ class AppTestAppSetTest:
                 return int(id(self) & 0x7fffffff)
         s = H()
         f = set([s])
-        print f
+        print(f)
         assert s in f
         f.remove(s)
         f.add(s)
@@ -519,7 +524,7 @@ class AppTestAppSetTest:
         key = set([2, 3])
         try:
             s.remove(key)
-        except KeyError, e:
+        except KeyError as e:
             assert e.args[0] is key
 
     def test_contains(self):
@@ -530,6 +535,39 @@ class AppTestAppSetTest:
             assert (c in s) == (c in word)
         raises(TypeError, s.__contains__, [])
 
+        logger = []
+
+        class Foo(object):
+
+            def __init__(self, value, name=None):
+                self.value = value
+                self.name = name or value
+
+            def __repr__(self):
+                return '<Foo %s>' % self.name
+
+            def __eq__(self, other):
+                logger.append((self, other))
+                return self.value == other.value
+
+            def __hash__(self):
+                return 42  # __eq__ will be used given all objects' hashes clash
+
+        foo1, foo2, foo3 = Foo(1), Foo(2), Foo(3)
+        foo42 = Foo(42)
+        foo_set = {foo1, foo2, foo3}
+        del logger[:]
+        foo42 in foo_set
+        logger_copy = set(logger[:])  # prevent re-evaluation during pytest error print
+        assert logger_copy == {(foo3, foo42), (foo2, foo42), (foo1, foo42)}
+
+        del logger[:]
+        foo2_bis = Foo(2, '2 bis')
+        foo2_bis in foo_set
+        logger_copy = set(logger[:])  # prevent re-evaluation during pytest error print
+        assert (foo2, foo2_bis) in logger_copy
+        assert logger_copy.issubset({(foo1, foo2_bis), (foo2, foo2_bis), (foo3, foo2_bis)})
+
     def test_remove(self):
         s = set('abc')
         s.remove('a')
@@ -548,12 +586,12 @@ class AppTestAppSetTest:
         for v1 in ['Q', (1,)]:
             try:
                 s.remove(v1)
-            except KeyError, e:
+            except KeyError as e:
                 v2 = e.args[0]
                 assert v1 == v2
             else:
                 assert False, 'Expected KeyError'
-        
+
     def test_singleton_empty_frozenset(self):
         class Frozenset(frozenset):
             pass
@@ -575,6 +613,40 @@ class AppTestAppSetTest:
             assert type(t) is subset
             assert not hasattr(t, 'x')
 
+    def test_reverse_ops(self):
+        assert set.__rxor__
+        assert frozenset.__rxor__
+        assert set.__ror__
+        assert frozenset.__ror__
+        assert set.__rand__
+        assert frozenset.__rand__
+        assert set.__rsub__
+        assert frozenset.__rsub__
+
+        # actual behaviour test
+        for base in [set, frozenset]:
+            class S(base):
+                def __xor__(self, other):
+                    if type(other) is not S:
+                        return NotImplemented
+                    return 1
+                __or__ = __and__ = __sub__ = __xor__
+            assert S([1, 2, 3]) ^ S([2, 3, 4]) == 1
+            assert S([1, 2, 3]) ^ {2, 3, 4} == {1, 4}
+            assert {1, 2, 3} ^ S([2, 3, 4]) == {1, 4}
+
+            assert S([1, 2, 3]) & S([2, 3, 4]) == 1
+            assert S([1, 2, 3]) & {2, 3, 4} == {2, 3}
+            assert {1, 2, 3} & S([2, 3, 4]) == {2, 3}
+
+            assert S([1, 2, 3]) | S([2, 3, 4]) == 1
+            assert S([1, 2, 3]) | {2, 3, 4} == {1, 2, 3, 4}
+            assert {1, 2, 3} | S([2, 3, 4]) == {1, 2, 3, 4}
+
+            assert S([1, 2, 3]) - S([2, 3, 4]) == 1
+            assert S([1, 2, 3]) - {2, 3, 4} == {1}
+            assert {1, 2, 3} - S([2, 3, 4]) == {1}
+
     def test_isdisjoint(self):
         assert set([1,2,3]).isdisjoint(set([4,5,6]))
         assert set([1,2,3]).isdisjoint(frozenset([4,5,6]))
@@ -1002,3 +1074,23 @@ class AppTestAppSetTest:
            raise ValueError
            yield 1
         raises(ValueError, set, f())
+
+    def test_frozenset_init_does_nothing(self):
+        f = frozenset([1, 2, 3])
+        f.__init__(4, 5, 6)
+        assert f == frozenset([1, 2, 3])
+
+    def test_error_message_wrong_self(self):
+        e = raises(TypeError, frozenset.copy, 42)
+        assert "frozenset" in str(e.value)
+        if hasattr(frozenset.copy, 'im_func'):
+            e = raises(TypeError, frozenset.copy.im_func, 42)
+            assert "'frozenset' object expected, got 'int' instead" in str(e.value)
+        if hasattr(set.copy, 'im_func'):
+            e = raises(TypeError, set.copy.im_func, 42)
+            assert "'set' object expected, got 'int' instead" in str(e.value)
+
+    def test_cant_mutate_frozenset_via_set(self):
+        x = frozenset()
+        raises(TypeError, set.add.im_func, x, 1)
+        raises(TypeError, set.__ior__.im_func, x, set([2]))
