@@ -331,9 +331,6 @@ class TranslationRemoveTypePtrTest(CCompiledMixin):
         try:
             res = self.meta_interp(main, [400])
             assert res == main(400)
-        except ConfigError as e:
-            assert str(e).startswith('invalid value asmgcc')
-            py.test.skip('asmgcc not supported')
         finally:
             del os.environ['PYPYLOG']
 
