@@ -4,6 +4,7 @@ indexed like a tuple but also exposes the st_xxx attributes.
 """
 
 import os, sys
+import collections
 
 from rpython.flowspace.model import Constant
 from rpython.flowspace.operation import op
@@ -20,13 +21,15 @@ from rpython.rlib._os_support import _preferred_traits, string_traits
 from rpython.rlib.objectmodel import specialize, we_are_translated, not_rpython
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
-from rpython.rlib.rarithmetic import intmask
+from rpython.rlib.rarithmetic import widen
 from rpython.rlib.rposix import (
     replace_os_function, handle_posix_error, _as_bytes0)
 from rpython.rlib import rposix
 
 _WIN32 = sys.platform.startswith('win')
 _LINUX = sys.platform.startswith('linux')
+_MACOS = sys.platform.startswith('darwin')
+_BSD = sys.platform.startswith('openbsd')
 
 if _WIN32:
     from rpython.rlib import rwin32
@@ -36,7 +39,7 @@ if _WIN32:
 # - ALL_STAT_FIELDS contains Float fields if the system can retrieve
 #   sub-second timestamps.
 # - TIMESPEC is defined when the "struct stat" contains st_atim field.
-if sys.platform.startswith('linux') or sys.platform.startswith('openbsd'):
+if _LINUX or _MACOS or _BSD:
     from rpython.rlib.rposix import TIMESPEC
 else:
     TIMESPEC = None
@@ -64,6 +67,10 @@ ALL_STAT_FIELDS = [
     ("nsec_mtime",   lltype.Signed),   #
     ("nsec_ctime",   lltype.Signed),   #
 ]
+if sys.platform == 'win32':
+    ALL_STAT_FIELDS.append(("st_file_attributes", lltype.Signed))
+    ALL_STAT_FIELDS.append(("st_reparse_tag", lltype.Signed))
+
 N_INDEXABLE_FIELDS = 10
 
 # For OO backends, expose only the portable fields (the first 10).
@@ -80,6 +87,7 @@ STATVFS_FIELDS = [
     ("f_favail", lltype.Signed),
     ("f_flag", lltype.Signed),
     ("f_namemax", lltype.Signed),
+    ("f_fsid", lltype.Unsigned),
 ]
 
 @specialize.arg(1)
@@ -142,27 +150,53 @@ class SomeStatResult(annmodel.SomeObject):
             TYPE = STAT_FIELD_TYPES[attrname]
         return lltype_to_annotation(TYPE)
 
-    def _get_rmarshall_support_(self):     # for rlib.rmarshal
-        # reduce and recreate stat_result objects from 10-tuples
-        # (we ignore the extra values here for simplicity and portability)
-        def stat_result_reduce(st):
-            return (st[0], st[1], st[2], st[3], st[4],
-                    st[5], st[6], st.st_atime, st.st_mtime, st.st_ctime)
+    if sys.platform == 'win32':
+        def _get_rmarshall_support_(self):     # for rlib.rmarshal
+            # reduce and recreate stat_result objects from 10-tuples
+            # (we ignore the extra values here for simplicity and portability)
+            def stat_result_reduce(st):
+                return (st[0], st[1], st[2], st[3], st[4],
+                        st[5], st[6], st.st_atime, st.st_mtime, st.st_ctime,
+                        st.st_file_attributes, st.st_reparse_tag)
 
-        def stat_result_recreate(tup):
-            atime, mtime, ctime = tup[7:]
-            result = tup[:7]
-            result += (int(atime), int(mtime), int(ctime))
-            result += extra_zeroes
-            result += (int((atime - result[7]) * 1e9),
-                       int((mtime - result[8]) * 1e9),
-                       int((ctime - result[9]) * 1e9))
-            return make_stat_result(result)
-        s_reduced = annmodel.SomeTuple([lltype_to_annotation(TYPE)
-                                    for name, TYPE in PORTABLE_STAT_FIELDS[:7]]
+            def stat_result_recreate(tup):
+                atime, mtime, ctime = tup[7:10]
+                result = tup[:7]
+                result += (int(atime), int(mtime), int(ctime))
+                result += extra_zeroes
+                result += (int((atime - result[7]) * 1e9),
+                           int((mtime - result[8]) * 1e9),
+                           int((ctime - result[9]) * 1e9))
+                result += tup[10:]
+                return make_stat_result(result)
+            s_reduced = annmodel.SomeTuple([lltype_to_annotation(TYPE)
+                                        for name, TYPE in PORTABLE_STAT_FIELDS[:7]]
+                                 + 3 * [lltype_to_annotation(lltype.Float)]
+                                 + 2 * [lltype_to_annotation(lltype.Int)])
+            extra_zeroes = (0,) * (len(STAT_FIELDS) - len(PORTABLE_STAT_FIELDS) - 3)
+            return s_reduced, stat_result_reduce, stat_result_recreate
+    else:
+        def _get_rmarshall_support_(self):     # for rlib.rmarshal
+            # reduce and recreate stat_result objects from 10-tuples
+            # (we ignore the extra values here for simplicity and portability)
+            def stat_result_reduce(st):
+                return (st[0], st[1], st[2], st[3], st[4],
+                        st[5], st[6], st.st_atime, st.st_mtime, st.st_ctime)
+
+            def stat_result_recreate(tup):
+                atime, mtime, ctime = tup[7:]
+                result = tup[:7]
+                result += (int(atime), int(mtime), int(ctime))
+                result += extra_zeroes
+                result += (int((atime - result[7]) * 1e9),
+                           int((mtime - result[8]) * 1e9),
+                           int((ctime - result[9]) * 1e9))
+                return make_stat_result(result)
+            s_reduced = annmodel.SomeTuple([lltype_to_annotation(TYPE)
+                                        for name, TYPE in PORTABLE_STAT_FIELDS[:7]]
                                  + 3 * [lltype_to_annotation(lltype.Float)])
-        extra_zeroes = (0,) * (len(STAT_FIELDS) - len(PORTABLE_STAT_FIELDS) - 3)
-        return s_reduced, stat_result_reduce, stat_result_recreate
+            extra_zeroes = (0,) * (len(STAT_FIELDS) - len(PORTABLE_STAT_FIELDS) - 3)
+            return s_reduced, stat_result_reduce, stat_result_recreate
 
 
 class __extend__(pairtype(SomeStatResult, annmodel.SomeInteger)):
@@ -251,9 +285,16 @@ def make_stat_result(tup):
         value = lltype.cast_primitive(TYPE, tup[i])
         positional.append(value)
     kwds = {}
-    kwds['st_atime'] = tup[7] + 1e-9 * tup[-3]
-    kwds['st_mtime'] = tup[8] + 1e-9 * tup[-2]
-    kwds['st_ctime'] = tup[9] + 1e-9 * tup[-1]
+    if sys.platform == 'win32':
+        kwds['st_atime'] = tup[7] + 1e-9 * tup[-5]
+        kwds['st_mtime'] = tup[8] + 1e-9 * tup[-4]
+        kwds['st_ctime'] = tup[9] + 1e-9 * tup[-3]
+        kwds['st_file_attributes'] = tup[-2]
+        kwds['st_reparse_tag'] = tup[-1]
+    else:
+        kwds['st_atime'] = tup[7] + 1e-9 * tup[-3]
+        kwds['st_mtime'] = tup[8] + 1e-9 * tup[-2]
+        kwds['st_ctime'] = tup[9] + 1e-9 * tup[-1]
     for value, (name, TYPE) in zip(tup, STAT_FIELDS)[N_INDEXABLE_FIELDS:]:
         if name.startswith('nsec_'):
             continue   # ignore the nsec_Xtime here
@@ -343,12 +384,12 @@ class __extend__(pairtype(StatvfsResultRepr, IntegerRepr)):
         index = s_int.const
         return r_sta.redispatch_getfield(hop, index)
 
-
 def make_statvfs_result(tup):
     args = tuple(
         lltype.cast_primitive(TYPE, value) for value, (name, TYPE) in
             zip(tup, STATVFS_FIELDS))
-    return os.statvfs_result(args)
+    # only used untranslated 
+    return statvfs_result(*args)
 
 class MakeStatvfsResultEntry(extregistry.ExtRegistryEntry):
     _about_ = make_statvfs_result
@@ -394,10 +435,15 @@ def posix_declaration(try_to_add=None):
     if TIMESPEC is not None:
 
         def _expand(lst, originalname, timespecname):
+            if _MACOS:  # fields are named e.g. st_atimespec
+                timespecname = originalname + "spec"
+            else:  # fields are named e.g. st_atim, with no e
+                timespecname = originalname[:-1]
+
             for i, (_name, _TYPE) in enumerate(lst):
                 if _name == originalname:
                     # replace the 'st_atime' field of type rffi.DOUBLE
-                    # with a field 'st_atim' of type 'struct timespec'
+                    # with the corresponding 'struct timespec' field
                     lst[i] = (timespecname, TIMESPEC)
                     break
 
@@ -443,7 +489,10 @@ if sys.platform != 'win32':
         posix_declaration(ALL_STAT_FIELDS[_i])
     del _i
 
-STAT_FIELDS += ALL_STAT_FIELDS[-3:]   # nsec_Xtime
+if sys.platform == 'win32':
+    STAT_FIELDS += ALL_STAT_FIELDS[-5:]   # nsec_Xtime, st_file_attributes, st_reparse_tag
+else:
+    STAT_FIELDS += ALL_STAT_FIELDS[-3:]   # nsec_Xtime
 
 # these two global vars only list the fields defined in the underlying platform
 STAT_FIELD_TYPES = dict(STAT_FIELDS)      # {'st_xxx': TYPE}
@@ -452,14 +501,19 @@ del _name, _TYPE
 
 STATVFS_FIELD_TYPES = dict(STATVFS_FIELDS)
 STATVFS_FIELD_NAMES = [name for name, tp in STATVFS_FIELDS]
-
+statvfs_result = collections.namedtuple('statvfs_result', STATVFS_FIELD_NAMES)
 
 def build_stat_result(st):
     # only for LL backends
     if TIMESPEC is not None:
-        atim = st.c_st_atim
-        mtim = st.c_st_mtim
-        ctim = st.c_st_ctim
+        if _MACOS:
+            atim = st.c_st_atimespec
+            mtim = st.c_st_mtimespec
+            ctim = st.c_st_ctimespec
+        else:
+            atim = st.c_st_atim
+            mtim = st.c_st_mtim
+            ctim = st.c_st_ctim
         atime, extra_atime = atim.c_tv_sec, int(atim.c_tv_nsec)
         mtime, extra_mtime = mtim.c_tv_sec, int(mtim.c_tv_nsec)
         ctime, extra_ctime = ctim.c_tv_sec, int(ctim.c_tv_nsec)
@@ -502,7 +556,8 @@ def build_statvfs_result(st):
         st.c_f_ffree,
         st.c_f_favail,
         st.c_f_flag,
-        st.c_f_namemax
+        st.c_f_namemax,
+        st.c_f_fsid,
     ))
 
 
@@ -549,13 +604,13 @@ def fstat(fd):
             return make_stat_result((win32traits._S_IFCHR,
                                      0, 0, 0, 0, 0,
                                      0, 0, 0, 0,
-                                     0, 0, 0))
+                                     0, 0, 0, 0, 0))
         elif filetype == win32traits.FILE_TYPE_PIPE:
             # socket or named pipe
             return make_stat_result((win32traits._S_IFIFO,
                                      0, 0, 0, 0, 0,
                                      0, 0, 0, 0,
-                                     0, 0, 0))
+                                     0, 0, 0, 0, 0))
         elif filetype == win32traits.FILE_TYPE_UNKNOWN:
             error = rwin32.GetLastError_saved()
             if error != 0:
@@ -563,16 +618,13 @@ def fstat(fd):
             # else: unknown but valid file
 
         # normal disk file (FILE_TYPE_DISK)
-        info = lltype.malloc(win32traits.BY_HANDLE_FILE_INFORMATION,
-                             flavor='raw', zero=True)
-        try:
-            res = win32traits.GetFileInformationByHandle(handle, info)
+        with lltype.scoped_alloc(win32traits.BY_HANDLE_FILE_INFORMATION,
+                             zero=True) as fileInfo:
+            res = win32traits.GetFileInformationByHandle(handle, fileInfo)
             if res == 0:
                 raise WindowsError(rwin32.GetLastError_saved(),
                                    "os_fstat failed")
-            return win32_by_handle_info_to_stat(win32traits, info)
-        finally:
-            lltype.free(info, flavor='raw')
+            return win32_by_handle_info_to_stat(win32traits, fileInfo, 0)
 
 @replace_os_function('stat')
 @specialize.argtype(0)
@@ -585,7 +637,7 @@ def stat(path):
     else:
         traits = _preferred_traits(path)
         path = traits.as_str0(path)
-        return win32_xstat(traits, path, traverse=True)
+        return win32_xstat3(traits, path, traverse=True)
 
 @replace_os_function('lstat')
 @specialize.argtype(0)
@@ -598,29 +650,29 @@ def lstat(path):
     else:
         traits = _preferred_traits(path)
         path = traits.as_str0(path)
-        return win32_xstat(traits, path, traverse=False)
+        return win32_xstat3(traits, path, traverse=False)
 
 @specialize.argtype(0)
 def stat3(path):
-    # On Windows, the algorithm behind os.stat() changed a lot between
-    # Python 2 and Python 3.  This is the Python 3 version.
-    if not _WIN32:
-        return stat(path)
-    else:
+    if _WIN32:
+        # On Windows, the algorithm behind os.stat() changed a lot between
+        # Python 2 and Python 3.  This is the Python 3 version.
         traits = _preferred_traits(path)
         path = traits.as_str0(path)
         return win32_xstat3(traits, path, traverse=True)
+    else:
+        return stat(path)
 
 @specialize.argtype(0)
 def lstat3(path):
-    # On Windows, the algorithm behind os.lstat() changed a lot between
-    # Python 2 and Python 3.  This is the Python 3 version.
-    if not _WIN32:
-        return lstat(path)
-    else:
+    if _WIN32:
+        # On Windows, the algorithm behind os.lstat() changed a lot between
+        # Python 2 and Python 3.  This is the Python 3 version.
         traits = _preferred_traits(path)
         path = traits.as_str0(path)
         return win32_xstat3(traits, path, traverse=False)
+    else:
+        return lstat(path)
 
 if rposix.HAVE_FSTATAT:
     from rpython.rlib.rposix import AT_FDCWD, AT_SYMLINK_NOFOLLOW
@@ -661,67 +713,149 @@ if _WIN32:
     def make_longlong(high, low):
         return (rffi.r_longlong(high) << 32) + rffi.r_longlong(low)
 
-    @specialize.arg(0)
-    def win32_xstat(traits, path, traverse=False):
-        # XXX 'traverse' is ignored
-        win32traits = make_win32_traits(traits)
-        with lltype.scoped_alloc(
-                win32traits.WIN32_FILE_ATTRIBUTE_DATA) as data:
-            res = win32traits.GetFileAttributesEx(
-                path, win32traits.GetFileExInfoStandard, data)
-            if res == 0:
-                errcode = rwin32.GetLastError_saved()
-                if errcode == win32traits.ERROR_SHARING_VIOLATION:
-                    res = win32_attributes_from_dir(
-                        win32traits, path, data)
-            if res == 0:
-                errcode = rwin32.GetLastError_saved()
-                raise WindowsError(errcode, "os_stat failed")
-            return win32_attribute_data_to_stat(win32traits, data)
+    def IsReparseTagNameSurrogate(_tag):
+        return widen(_tag) & 0x20000000
 
     @specialize.arg(0)
-    def win32_xstat3(traits, path, traverse=False):
+    def win32_xstat3(traits, path0, traverse=False):
         # This is the Python3 version of os.stat() or lstat().
-        # XXX 'traverse' is ignored, and everything related to
-        # the "reparse points" is missing
         win32traits = make_win32_traits(traits)
+        path = traits.as_str0(path0)
 
-        hFile = win32traits.CreateFile(traits.as_str0(path),
-            win32traits.FILE_READ_ATTRIBUTES,
-            0,
+        with lltype.scoped_alloc(win32traits.BY_HANDLE_FILE_INFORMATION,
+                                 zero=True) as fileInfo:
+            with lltype.scoped_alloc(win32traits.FILE_ATTRIBUTE_TAG_INFO,
+                                     zero=True) as tagInfo:
+                return win32_xstat_impl(win32traits, path, traverse, fileInfo, tagInfo)
+
+    @specialize.arg(0)
+    def win32_xstat_impl(traits, path, traverse, fileInfo, tagInfo):
+        access = traits.FILE_READ_ATTRIBUTES
+        flags = traits.FILE_FLAG_BACKUP_SEMANTICS
+        isUnhandledTag = False
+        if not traverse:
+            flags |= traits.FILE_FLAG_OPEN_REPARSE_POINT
+        hFile = traits.CreateFile(path, access, 0,
             lltype.nullptr(rwin32.LPSECURITY_ATTRIBUTES.TO),
-            win32traits.OPEN_EXISTING,
-            win32traits.FILE_ATTRIBUTE_NORMAL |
-                win32traits.FILE_FLAG_BACKUP_SEMANTICS |
-                0,  # win32traits.FILE_FLAG_OPEN_REPARSE_POINT,
+            traits.OPEN_EXISTING,
+            flags,
             rwin32.NULL_HANDLE)
 
         if hFile == rwin32.INVALID_HANDLE_VALUE:
+            # Either the path doesn't exist, or the caller lacks access
             errcode = rwin32.GetLastError_saved()
-            if (errcode != win32traits.ERROR_ACCESS_DENIED and
-                errcode != win32traits.ERROR_SHARING_VIOLATION):
-                raise WindowsError(errcode, "os_stat failed")
-
-            with lltype.scoped_alloc(
-                    win32traits.WIN32_FILE_ATTRIBUTE_DATA) as data:
-                if win32_attributes_from_dir(win32traits, path, data) == 0:
+            if (errcode == traits.ERROR_ACCESS_DENIED or
+                errcode == traits.ERROR_SHARING_VIOLATION):
+                # Try reading the parent directory 
+                if win32_attributes_from_dir(traits, path, fileInfo, tagInfo) == 0:
                     raise WindowsError(rwin32.GetLastError_saved(),
                                        "win32_attributes_from_dir failed")
-                return win32_attribute_data_to_stat(win32traits, data)
+                if widen(fileInfo.c_dwFileAttributes) & traits.FILE_ATTRIBUTE_REPARSE_POINT:
+                    if traverse or not IsReparseTagNameSurrogate(tagInfo.c_ReparseTag):
+                        raise WindowsError(rwin32.GetLastError_saved(),
+                                           "win32_xstat failed")
+            elif errcode == traits.ERROR_INVALID_PARAMETER:
+                # \\.\con requires read or write access.
+                hFile = traits.CreateFile(path,
+                            access | traits.GENERIC_READ,
+                            traits.FILE_SHARE_READ | traits.FILE_SHARE_WRITE,
+                            lltype.nullptr(rwin32.LPSECURITY_ATTRIBUTES.TO),
+                            traits.OPEN_EXISTING, flags,
+                            rwin32.NULL_HANDLE)
+                if hFile == rwin32.INVALID_HANDLE_VALUE:
+                    raise WindowsError(rwin32.GetLastError_saved(),
+                                       "win32_xstat failed")
+            elif errcode == traits.ERROR_CANT_ACCESS_FILE:
+                # bpo37834: opne unhandled reparse points if traverse fails
+                if traverse:
+                    traverse = False
+                    isUnhandledTag = True
+                    hFile = traits.CreateFile(path, access, 0,
+                        lltype.nullptr(rwin32.LPSECURITY_ATTRIBUTES.TO),
+                        traits.OPEN_EXISTING,
+                        flags | traits.FILE_FLAG_OPEN_REPARSE_POINT,
+                        rwin32.NULL_HANDLE)
+                if hFile == rwin32.INVALID_HANDLE_VALUE:
+                    raise WindowsError(rwin32.GetLastError_saved(),
+                                       "win32_xstat failed")
+            else:
+                raise WindowsError(errcode, "os_stat failed")
+        
+        if hFile != rwin32.INVALID_HANDLE_VALUE:
+            # Handle types other than files on disk.
+            fileType = traits.GetFileType(hFile)
+            if fileType != traits.FILE_TYPE_DISK:
+                errcode = rwin32.GetLastError_saved()
+                if fileType == traits.FILE_TYPE_UNKNOWN and errcode != 0:
+                    rwin32.CloseHandle(hFile)
+                    raise WindowsError(errcode, "os_stat failed")
+                fileAttributes = widen(traits.GetFileAttributes(path))
+                st_mode = 0
+                if (fileAttributes != traits.INVALID_FILE_ATTRIBUTES and
+                        fileAttributes & traits.FILE_ATTRIBUTE_DIRECTORY):
+                    # \\.\pipe\ or \\.\mailslot\
+                    st_mode = traits._S_IFDIR
+                elif fileType == traits.FILE_TYPE_CHAR:
+                    # \\.\nul
+                    st_mode = traits._S_IFCHR
+                elif fileType == traits.FILE_TYPE_PIPE:
+                    # \\.\pipe\spam
+                    st_mode = traits._S_IFIFO
+                rwin32.CloseHandle(hFile)
+                result = (st_mode,
+                  0, 0, 0, 0, 0,
+                  0,
+                  0, 0, 0,
+                  0, 0, 0,
+                  0, 0)
+                # FILE_TYPE_UNKNOWN, e.g. \\.\mailslot\waitfor.exe\spam
+                return make_stat_result(result)
+            # Query the reparse tag, and traverse a non-link.
+            if not traverse:
+                if not traits.GetFileInformationByHandleEx(hFile,
+                            traits.FileAttributeTagInfo, tagInfo,
+                            traits.TagInfoSize):
+                    errcode = rwin32.GetLastError_saved()
+                    if errcode in (traits.ERROR_INVALID_PARAMETER,
+                                   traits.ERROR_INVALID_FUNCTION,
+                                   traits.ERROR_NOT_SUPPORTED):
+                        tagInfo.c_FileAttributes = rffi.cast(
+                                    rwin32.DWORD, traits.FILE_ATTRIBUTE_NORMAL)
+                        tagInfo.c_ReparseTag = rffi.cast(rwin32.DWORD, 0)
+                    else:
+                        rwin32.CloseHandle(hFile)
+                        raise WindowsError(errcode, "os_stat failed")
+                elif widen(tagInfo.c_FileAttributes) & traits.FILE_ATTRIBUTE_REPARSE_POINT:
+                    if IsReparseTagNameSurrogate(tagInfo.c_ReparseTag):
+                        if isUnhandledTag:
+                            # Traversing previously failed for either this
+                            # link or its target.
+                            rwin32.CloseHandle(hFile)
+                            raise WindowsError(
+                                traits.ERROR_CANT_ACCESS_FILE,
+                                "os_stat failed")
+                    # Traverse a non-link, but not if traversing already
+                    # failed for an unhandled tag.
+                    elif not isUnhandledTag:
+                        rwin32.CloseHandle(hFile)
+                        return win32_xstat_impl(traits, path, True, fileInfo, tagInfo)
 
-        with lltype.scoped_alloc(
-                win32traits.BY_HANDLE_FILE_INFORMATION, zero=True) as data:
-            res = win32traits.GetFileInformationByHandle(hFile, data)
-            errcode = rwin32.GetLastError_saved()
-            rwin32.CloseHandle(hFile)
-            if res == 0:
-                raise WindowsError(errcode, "GetFileInformationByHandle failed")
-            return win32_by_handle_info_to_stat(win32traits, data)
+        res = traits.GetFileInformationByHandle(hFile, fileInfo)
+        errcode = rwin32.GetLastError_saved()
+        rwin32.CloseHandle(hFile)
+        if res == 0:
+            raise WindowsError(errcode, "GetFileInformationByHandle failed")
+        result = win32_by_handle_info_to_stat(traits, fileInfo, tagInfo.c_ReparseTag)
+        
+        # TBD: adjust the file execute permissions by finding the file extension
+        # if fileExtension in ('exe', 'bat', 'cmd', 'com'):
+        #    result.st_mode |= 0x0111
+        return result
 
     @specialize.arg(0)
     def win32_attributes_to_mode(win32traits, attributes):
         m = 0
-        attributes = intmask(attributes)
+        attributes = widen(attributes)
         if attributes & win32traits.FILE_ATTRIBUTE_DIRECTORY:
             m |= win32traits._S_IFDIR | 0111 # IFEXEC for user,group,other
         else:
@@ -740,16 +874,23 @@ if _WIN32:
         mtime, extra_mtime = FILE_TIME_to_time_t_nsec(info.c_ftLastWriteTime)
         atime, extra_atime = FILE_TIME_to_time_t_nsec(info.c_ftLastAccessTime)
 
+        st_ino = 0
+        st_dev = 0
+        st_nlink = 0
+        st_file_attributes = info.c_dwFileAttributes
+        st_reparse_tag = 0
+
         result = (st_mode,
-                  0, 0, 0, 0, 0,
+                  st_ino, st_dev, st_nlink, 0, 0,
                   st_size,
                   atime, mtime, ctime,
-                  extra_atime, extra_mtime, extra_ctime)
+                  extra_atime, extra_mtime, extra_ctime,
+                  st_file_attributes, st_reparse_tag)
 
         return make_stat_result(result)
 
     @specialize.arg(0)
-    def win32_by_handle_info_to_stat(win32traits, info):
+    def win32_by_handle_info_to_stat(win32traits, info, reparse_tag):
         # similar to the one above
         st_mode = win32_attributes_to_mode(win32traits, info.c_dwFileAttributes)
         st_size = make_longlong(info.c_nFileSizeHigh, info.c_nFileSizeLow)
@@ -761,29 +902,37 @@ if _WIN32:
         st_ino = make_longlong(info.c_nFileIndexHigh, info.c_nFileIndexLow)
         st_dev = info.c_dwVolumeSerialNumber
         st_nlink = info.c_nNumberOfLinks
+        st_file_attributes = info.c_dwFileAttributes
+        st_reparse_tag = reparse_tag
 
         result = (st_mode,
                   st_ino, st_dev, st_nlink, 0, 0,
                   st_size,
                   atime, mtime, ctime,
-                  extra_atime, extra_mtime, extra_ctime)
+                  extra_atime, extra_mtime, extra_ctime,
+                  st_file_attributes, st_reparse_tag)
 
         return make_stat_result(result)
 
     @specialize.arg(0)
-    def win32_attributes_from_dir(win32traits, path, data):
-        filedata = lltype.malloc(win32traits.WIN32_FIND_DATA, flavor='raw')
-        try:
-            hFindFile = win32traits.FindFirstFile(path, filedata)
+    def win32_attributes_from_dir(traits, path, info, tagInfo):
+        with lltype.scoped_alloc(traits.WIN32_FIND_DATA) as filedata:
+            hFindFile = traits.FindFirstFile(path, filedata)
             if hFindFile == rwin32.INVALID_HANDLE_VALUE:
                 return 0
-            win32traits.FindClose(hFindFile)
-            data.c_dwFileAttributes = filedata.c_dwFileAttributes
-            rffi.structcopy(data.c_ftCreationTime, filedata.c_ftCreationTime)
-            rffi.structcopy(data.c_ftLastAccessTime, filedata.c_ftLastAccessTime)
-            rffi.structcopy(data.c_ftLastWriteTime, filedata.c_ftLastWriteTime)
-            data.c_nFileSizeHigh    = filedata.c_nFileSizeHigh
-            data.c_nFileSizeLow     = filedata.c_nFileSizeLow
+            traits.FindClose(hFindFile)
+            tagInfo.c_ReparseTag = win32_find_data_to_file_info(traits, filedata, info)
             return 1
-        finally:
-            lltype.free(filedata, flavor='raw')
+
+    @specialize.arg(0)
+    def win32_find_data_to_file_info(traits, filedata, info):
+        info.c_dwFileAttributes = filedata.c_dwFileAttributes
+        rffi.structcopy(info.c_ftCreationTime, filedata.c_ftCreationTime)
+        rffi.structcopy(info.c_ftLastAccessTime, filedata.c_ftLastAccessTime)
+        rffi.structcopy(info.c_ftLastWriteTime, filedata.c_ftLastWriteTime)
+        info.c_nFileSizeHigh    = filedata.c_nFileSizeHigh
+        info.c_nFileSizeLow     = filedata.c_nFileSizeLow
+        attr = widen(filedata.c_dwFileAttributes)
+        if attr & traits.FILE_ATTRIBUTE_REPARSE_POINT:
+            return filedata.c_dwReserved0
+        return 0
