@@ -25,10 +25,12 @@ class TestFile(BaseRtypingTest):
             f.close()
 
         f()
-        assert open(fname, "r").read() == "dupa"
+        with open(fname, "r") as fid:
+            assert fid.read() == "dupa"
         os.unlink(fname)
         self.interpret(f, [])
-        assert open(fname, "r").read() == "dupa"
+        with open(fname, "r") as fid:
+            assert fid.read() == "dupa"
 
     def test_open_errors(self):
         def f(run):
@@ -57,8 +59,8 @@ class TestFile(BaseRtypingTest):
                 assert False
 
             try:
-                os.fdopen(42, "badmode")
-            except ValueError:
+                rfile.create_fdopen_rfile(42, "badmode")
+            except  ValueError:
                 pass
             else:
                 assert False
@@ -325,10 +327,12 @@ class TestFile(BaseRtypingTest):
             f2.close()
 
         f()
-        assert open(fname).read() == "xxx"
+        with open(fname) as fid:
+            assert fid.read() == "xxx"
         os.unlink(fname)
         self.interpret(f, [])
-        assert open(fname).read() == "xxx"
+        with open(fname) as fid:
+            assert fid.read() == "xxx"
 
     def test_fileno(self):
         fname = str(self.tmpdir.join('file_5'))
@@ -422,10 +426,12 @@ class TestFile(BaseRtypingTest):
                 assert False
 
         f()
-        assert open(fname, "r").read() == "dupa"
+        with open(fname) as fid:
+            assert fid.read() == "dupa"
         os.unlink(fname)
         self.interpret(f, [])
-        assert open(fname, "r").read() == "dupa"
+        with open(fname) as fid:
+            assert fid.read() == "dupa"
 
 
 class TestDirect:
@@ -470,7 +476,8 @@ class TestDirect:
                 s = ''.join([chr(32+(k&63)) for k in range(j, j + i)])
                 j += 1
                 print >> f, s
-        expected = open(fname).readlines()
+        with open(fname) as fid:
+            expected = fid.readlines()
         expected += ['', '']
         assert len(expected) == 252
 
@@ -502,7 +509,7 @@ class TestPopen(object):
             py.test.skip("not for win32")
 
     def test_popen(self):
-        f = rfile.create_popen_file("python -c 'print 42'", "r")
+        f = rfile.create_popen_file("python -c 'print(42)'", "r")
         s = f.read()
         f.close()
         assert s == '42\n'
@@ -510,7 +517,7 @@ class TestPopen(object):
     def test_pclose(self):
         retval = 32
         printval = 42
-        cmd = "python -c 'import sys; print %s; sys.exit(%s)'" % (
+        cmd = "python -c 'import sys; print(%s); sys.exit(%s)'" % (
             printval, retval)
         f = rfile.create_popen_file(cmd, "r")
         s = f.read()
@@ -526,7 +533,7 @@ class TestPopenR(BaseRtypingTest):
 
     def test_popen(self):
         printval = 42
-        cmd = "python -c 'print %s'" % printval
+        cmd = "python -c 'print(%s)'" % printval
         def f():
             f = rfile.create_popen_file(cmd, "r")
             s = f.read()
@@ -537,7 +544,7 @@ class TestPopenR(BaseRtypingTest):
     def test_pclose(self):
         printval = 42
         retval = 32
-        cmd = "python -c 'import sys; print %s; sys.exit(%s)'" % (
+        cmd = "python -c 'import sys; print(%s); sys.exit(%s)'" % (
             printval, retval)
         def f():
             f = rfile.create_popen_file(cmd, "r")
