@@ -1204,7 +1204,7 @@ if sys.platform == 'darwin':
 else:
     _findLib_gcc_fallback = lambda name: None
 
-def get_ctypes_callable(funcptr, calling_conv, natural_arity):
+def get_ctypes_callable(funcptr, calling_conv):
     if not ctypes:
         raise ImportError("ctypes is needed to use ll2ctypes")
 
@@ -1297,16 +1297,10 @@ def get_ctypes_callable(funcptr, calling_conv, natural_arity):
 
     # get_ctypes_type() can raise NotImplementedError too
     from rpython.rtyper.lltypesystem import rffi
-    argtypes = [get_ctypes_type(T) if T is not rffi.VOIDP
+    cfunc.argtypes = [get_ctypes_type(T) if T is not rffi.VOIDP
                                          else ctypes.c_void_p
                       for T in FUNCTYPE.ARGS
                       if not T is lltype.Void]
-    if natural_arity != -1:
-        cfunc.argtypes = argtypes[:natural_arity]
-        cfunc.extraargs = argtypes[natural_arity:]
-    else:
-        cfunc.argtypes = argtypes
-        cfunc.extraargs = []
     if FUNCTYPE.RESULT is lltype.Void:
         cfunc.restype = None
     else:
@@ -1316,29 +1310,26 @@ def get_ctypes_callable(funcptr, calling_conv, natural_arity):
 class LL2CtypesCallable(object):
     # a special '_callable' object that invokes ctypes
 
-    def __init__(self, FUNCTYPE, calling_conv, natural_arity):
+    def __init__(self, FUNCTYPE, calling_conv):
         self.FUNCTYPE = FUNCTYPE
         self.calling_conv = calling_conv
         self.trampoline = None
-        self.natural_arity = natural_arity
         #self.funcptr = ...  set later
 
     def __call__(self, *argvalues):
         with rlock:
             if self.trampoline is None:
                 # lazily build the corresponding ctypes function object
-                cfunc = get_ctypes_callable(self.funcptr, self.calling_conv,
-                                            self.natural_arity)
-                self.trampoline = get_ctypes_trampoline(self.FUNCTYPE, cfunc, self.natural_arity)
+                cfunc = get_ctypes_callable(self.funcptr, self.calling_conv)
+                self.trampoline = get_ctypes_trampoline(self.FUNCTYPE, cfunc)
         # perform the call
         return self.trampoline(*argvalues)
 
     def get_real_address(self):
-        cfunc = get_ctypes_callable(self.funcptr, self.calling_conv,
-                                    self.natural_arity)
+        cfunc = get_ctypes_callable(self.funcptr, self.calling_conv)
         return ctypes.cast(cfunc, ctypes.c_void_p).value
 
-def get_ctypes_trampoline(FUNCTYPE, cfunc, natural_arity=-1):
+def get_ctypes_trampoline(FUNCTYPE, cfunc):
     RESULT = FUNCTYPE.RESULT
     container_arguments = []
     for i in range(len(FUNCTYPE.ARGS)):
@@ -1358,8 +1349,6 @@ def get_ctypes_trampoline(FUNCTYPE, cfunc, natural_arity=-1):
                 cvalue = lltype2ctypes(argvalues[i])
                 if i in container_arguments:
                     cvalue = cvalue.contents
-                if natural_arity > 0 and i >= natural_arity:
-                    cvalue = cfunc.extraargs[i - natural_arity](cvalue)
                 cargs.append(cvalue)
         _callback_exc_info = None
         _restore_c_errno()
