@@ -7,7 +7,7 @@ from rpython.rtyper.lltypesystem.llmemory import cast_ptr_to_adr
 from rpython.rtyper.lltypesystem.llmemory import itemoffsetof
 from rpython.rtyper.llannotation import lltype_to_annotation
 from rpython.tool.sourcetools import func_with_new_name
-from rpython.rlib.objectmodel import Symbolic
+from rpython.rlib.objectmodel import Symbolic, specialize, not_rpython
 from rpython.rlib.objectmodel import keepalive_until_here, enforceargs
 from rpython.rlib import rarithmetic, rgc
 from rpython.rtyper.extregistry import ExtRegistryEntry
@@ -74,7 +74,7 @@ RFFI_ALT_ERRNO           = 64    # read, save using alt tl destination
 def llexternal(name, args, result, _callable=None,
                compilation_info=ExternalCompilationInfo(),
                sandboxsafe=False, releasegil='auto',
-               _nowrapper=False, calling_conv='c',
+               _nowrapper=False, calling_conv=None,
                elidable_function=False, macro=None,
                random_effects_on_gcobjs='auto',
                save_err=RFFI_ERR_NONE):
@@ -96,7 +96,17 @@ def llexternal(name, args, result, _callable=None,
                 we consider that the function is really short-running and
                 don't bother releasing the GIL.  An explicit True or False
                 overrides this logic.
+
+    calling_conv: if 'unknown' or 'win', the C function is not directly seen
+                  by the JIT.  If 'c', it can be seen (depending on
+                  releasegil=False).  For tests only, or if _nowrapper,
+                  it defaults to 'c'.
     """
+    if calling_conv is None:
+        if sys.platform == 'win32' and not _nowrapper:
+            calling_conv = 'unknown'
+        else:
+            calling_conv = 'c'
     if _callable is not None:
         assert callable(_callable)
     ext_type = lltype.FuncType(args, result)
@@ -107,7 +117,8 @@ def llexternal(name, args, result, _callable=None,
             _callable = generate_macro_wrapper(
                 name, macro, ext_type, compilation_info)
         else:
-            _callable = ll2ctypes.LL2CtypesCallable(ext_type, calling_conv)
+            _callable = ll2ctypes.LL2CtypesCallable(ext_type,
+                'c' if calling_conv == 'unknown' else calling_conv)
     else:
         assert macro is None, "'macro' is useless if you specify '_callable'"
     if elidable_function:
@@ -134,6 +145,9 @@ def llexternal(name, args, result, _callable=None,
         # Also, _nowrapper functions cannot release the GIL, by default.
         invoke_around_handlers = not sandboxsafe and not _nowrapper
 
+    if _nowrapper and isinstance(_callable, ll2ctypes.LL2CtypesCallable):
+        kwds['_real_integer_addr'] = _callable.get_real_address
+
     if random_effects_on_gcobjs not in (False, True):
         random_effects_on_gcobjs = (
             invoke_around_handlers or   # because it can release the GIL
@@ -176,7 +190,14 @@ def llexternal(name, args, result, _callable=None,
                 if %(save_err)d:
                     from rpython.rlib import rposix
                     rposix._errno_before(%(save_err)d)
-                res = funcptr(%(argnames)s)
+                if we_are_translated():
+                    res = funcptr(%(argnames)s)
+                else:
+                    try:    # only when non-translated
+                        res = funcptr(%(argnames)s)
+                    except:
+                        rgil.acquire()
+                        raise
                 if %(save_err)d:
                     from rpython.rlib import rposix
                     rposix._errno_after(%(save_err)d)
@@ -185,6 +206,7 @@ def llexternal(name, args, result, _callable=None,
         """ % locals())
         miniglobals = {'funcptr':     funcptr,
                        '__name__':    __name__, # for module name propagation
+                       'we_are_translated': we_are_translated,
                        }
         exec source.compile() in miniglobals
         call_external_function = miniglobals['call_external_function']
@@ -205,11 +227,17 @@ def llexternal(name, args, result, _callable=None,
     else:
         # if we don't have to invoke the GIL handling, we can just call
         # the low-level function pointer carelessly
-        if macro is None and save_err == RFFI_ERR_NONE:
+        # ...well, unless it's a macro, in which case we still have
+        # to hide it from the JIT...
+        need_wrapper = (macro is not None or save_err != RFFI_ERR_NONE)
+        # ...and unless we're on Windows and the calling convention is
+        # 'win' or 'unknown'
+        if calling_conv != 'c':
+            need_wrapper = True
+        #
+        if not need_wrapper:
             call_external_function = funcptr
         else:
-            # ...well, unless it's a macro, in which case we still have
-            # to hide it from the JIT...
             argnames = ', '.join(['a%d' % i for i in range(len(args))])
             source = py.code.Source("""
                 def call_external_function(%(argnames)s):
@@ -249,10 +277,11 @@ def llexternal(name, args, result, _callable=None,
             arg = args[i]
             if TARGET == CCHARP or TARGET is VOIDP:
                 if arg is None:
+                    from rpython.rtyper.annlowlevel import llstr
                     arg = lltype.nullptr(CCHARP.TO)   # None => (char*)NULL
-                    to_free = to_free + (arg, '\x04')
+                    to_free = to_free + (arg, llstr(None), '\x04')
                 elif isinstance(arg, str):
-                    tup = get_nonmovingbuffer_final_null(arg)
+                    tup = get_nonmovingbuffer_ll_final_null(arg)
                     to_free = to_free + tup
                     arg = tup[0]
                 elif isinstance(arg, unicode):
@@ -286,10 +315,10 @@ def llexternal(name, args, result, _callable=None,
             arg = args[i]
             if TARGET == CCHARP or TARGET is VOIDP:
                 if arg is None:
-                    to_free = to_free[2:]
+                    to_free = to_free[3:]
                 elif isinstance(arg, str):
-                    free_nonmovingbuffer(arg, to_free[0], to_free[1])
-                    to_free = to_free[2:]
+                    free_nonmovingbuffer_ll(to_free[0], to_free[1], to_free[2])
+                    to_free = to_free[3:]
             elif TARGET == CWCHARP:
                 if arg is None:
                     to_free = to_free[1:]
@@ -308,10 +337,6 @@ def llexternal(name, args, result, _callable=None,
     # for debugging, stick ll func ptr to that
     wrapper._ptr = funcptr
     wrapper = func_with_new_name(wrapper, name)
-
-    if calling_conv != "c":
-        wrapper = jit.dont_look_inside(wrapper)
-
     return wrapper
 
 
@@ -340,10 +365,9 @@ def _make_wrapper_for(TP, callable, callbackholder, use_gil):
 
         def wrapper(%(args)s):    # no *args - no GIL for mallocing the tuple
             if rgil is not None:
-                rgil.acquire()
+                rgil.acquire_maybe_in_new_thread()
             # from now on we hold the GIL
-            stackcounter.stacks_counter += 1
-            llop.gc_stack_bottom(lltype.Void)   # marker for trackgcroot.py
+            llop.gc_stack_bottom(lltype.Void)   # marker to enter RPython from C
             try:
                 result = callable(%(args)s)
             except Exception, e:
@@ -354,7 +378,6 @@ def _make_wrapper_for(TP, callable, callbackholder, use_gil):
                     import traceback
                     traceback.print_exc()
                 result = errorcode
-            stackcounter.stacks_counter -= 1
             if rgil is not None:
                 rgil.release()
             # here we don't hold the GIL any more. As in the wrapper() produced
@@ -366,20 +389,12 @@ def _make_wrapper_for(TP, callable, callbackholder, use_gil):
     miniglobals['Exception'] = Exception
     miniglobals['os'] = os
     miniglobals['we_are_translated'] = we_are_translated
-    miniglobals['stackcounter'] = stackcounter
     exec source.compile() in miniglobals
     return miniglobals['wrapper']
 _make_wrapper_for._annspecialcase_ = 'specialize:memo'
 
 AroundFnPtr = lltype.Ptr(lltype.FuncType([], lltype.Void))
 
-class StackCounter:
-    def _cleanup_(self):
-        self.stacks_counter = 0     # number of "stack pieces": callbacks
-                                    # and threads increase it by one
-
-stackcounter = StackCounter()
-stackcounter._cleanup_()
 
 def llexternal_use_eci(compilation_info):
     """Return a dummy function that, if called in a RPython program,
@@ -490,7 +505,7 @@ TYPES += ['signed char', 'unsigned char',
 # This is a bit of a hack since we can't use rffi_platform here.
 try:
     sizeof_c_type('__int128_t', ignore_errors=True)
-    TYPES += ['__int128_t']
+    TYPES += ['__int128_t', '__uint128_t']
 except CompilationError:
     pass
 
@@ -510,9 +525,10 @@ if os.name != 'nt':
           'int_fast64_t', 'uint_fast64_t',
           'intmax_t', 'uintmax_t'])
 else:
-    MODE_T = lltype.Signed
+    # MODE_T is set later
     PID_T = lltype.Signed
     SSIZE_T = lltype.Signed
+    PTRDIFF_T = lltype.Signed
 
 def populate_inttypes():
     names = []
@@ -522,7 +538,10 @@ def populate_inttypes():
         if name.startswith('unsigned'):
             name = 'u' + name[9:]
             signed = False
-        elif name == 'size_t' or name.startswith('uint'):
+        elif (name == 'size_t' or name.startswith('uint')
+                               or name.startswith('__uint')):
+            signed = False
+        elif name == 'wchar_t' and sys.platform == 'win32':
             signed = False
         else:
             signed = True
@@ -577,6 +596,9 @@ NUMBER_TYPES.append(INT_real)
 # 'int' type, whereas rarithmetic.r_int corresponds to the
 # Python-level int type (which is a C long).  Fun.
 
+if os.name == 'nt':
+    MODE_T = INT
+
 def CStruct(name, *fields, **kwds):
     """ A small helper to create external C structure, not the
     pypy one
@@ -739,7 +761,13 @@ LONGDOUBLEP = lltype.Ptr(lltype.Array(LONGDOUBLE, hints={'nolength': True}))
 
 # Signed, Signed *
 SIGNED = lltype.Signed
-SIGNEDP = lltype.Ptr(lltype.Array(SIGNED, hints={'nolength': True}))
+SIGNEDP = lltype.Ptr(lltype.Array(lltype.Signed, hints={'nolength': True}))
+SIGNEDPP = lltype.Ptr(lltype.Array(SIGNEDP, hints={'nolength': True}))
+
+# Unsigned, Unsigned *
+UNSIGNED = lltype.Unsigned
+UNSIGNEDP = lltype.Ptr(lltype.Array(lltype.Unsigned, hints={'nolength': True}))
+
 
 # various type mapping
 
@@ -791,6 +819,7 @@ def make_string_mappings(strtype):
             lltype.free(cp, flavor='raw', track_allocation=True)
         else:
             lltype.free(cp, flavor='raw', track_allocation=False)
+    free_charp._annenforceargs_ = [None, bool]
 
     # str -> already-existing char[maxsize]
     def str2chararray(s, array, maxsize):
@@ -809,85 +838,101 @@ def make_string_mappings(strtype):
     # char* -> str
     # doesn't free char*
     def charp2str(cp):
+        if not we_are_translated():
+            res = []
+            size = 0
+            while True:
+                c = cp[size]
+                if c == lastchar:
+                    return assert_str0("".join(res))
+                res.append(c)
+                size += 1
+
         size = 0
         while cp[size] != lastchar:
             size += 1
         return assert_str0(charpsize2str(cp, size))
     charp2str._annenforceargs_ = [lltype.SomePtr(TYPEP)]
 
-    # str -> char*, bool, bool
+    # str -> (buf, llobj, flag)
     # Can't inline this because of the raw address manipulation.
     @jit.dont_look_inside
-    def get_nonmovingbuffer(data):
+    def get_nonmovingbuffer_ll(data):
         """
         Either returns a non-moving copy or performs neccessary pointer
         arithmetic to return a pointer to the characters of a string if the
         string is already nonmovable or could be pinned.  Must be followed by a
-        free_nonmovingbuffer call.
+        free_nonmovingbuffer_ll call.
 
-        Also returns a char:
-         * \4: no pinning, returned pointer is inside 'data' which is nonmovable
-         * \5: 'data' was pinned, returned pointer is inside
+        The return type is a 3-tuple containing the "char *" result,
+        a pointer to the low-level string object, and a flag as a char:
+
+         * \4: no pinning, returned pointer is inside nonmovable 'llobj'
+         * \5: 'llobj' was pinned, returned pointer is inside
          * \6: pinning failed, returned pointer is raw malloced
 
         For strings (not unicodes), the len()th character of the resulting
         raw buffer is available, but not initialized.  Use
-        get_nonmovingbuffer_final_null() instead of get_nonmovingbuffer()
+        get_nonmovingbuffer_ll_final_null() instead of get_nonmovingbuffer_ll()
         to get a regular null-terminated "char *".
         """
 
-        lldata = llstrtype(data)
+        llobj = llstrtype(data)
         count = len(data)
 
-        if we_are_translated_to_c() and not rgc.can_move(data):
-            flag = '\x04'
+        if rgc.must_split_gc_address_space():
+            flag = '\x06'    # always make a copy in this case
+        elif we_are_translated_to_c() and not rgc.can_move(llobj):
+            flag = '\x04'    # no copy needed
         else:
-            if we_are_translated_to_c() and rgc.pin(data):
-                flag = '\x05'
+            if we_are_translated_to_c() and rgc.pin(llobj):
+                flag = '\x05'     # successfully pinned
             else:
-                buf = lltype.malloc(TYPEP.TO, count + (TYPEP is CCHARP),
-                                    flavor='raw')
-                copy_string_to_raw(lldata, buf, 0, count)
-                return buf, '\x06'
-                # ^^^ raw malloc used to get a nonmovable copy
+                flag = '\x06'     # must still make a copy
+        if flag == '\x06':
+            buf = lltype.malloc(TYPEP.TO, count + (TYPEP is CCHARP),
+                                flavor='raw')
+            copy_string_to_raw(llobj, buf, 0, count)
+            return buf, llobj, '\x06'
+            # ^^^ raw malloc used to get a nonmovable copy
         #
         # following code is executed after we're translated to C, if:
         # - rgc.can_move(data) and rgc.pin(data) both returned true
         # - rgc.can_move(data) returned false
-        data_start = cast_ptr_to_adr(lldata) + \
+        data_start = cast_ptr_to_adr(llobj) + \
             offsetof(STRTYPE, 'chars') + itemoffsetof(STRTYPE.chars, 0)
 
-        return cast(TYPEP, data_start), flag
+        return cast(TYPEP, data_start), llobj, flag
         # ^^^ already nonmovable. Therefore it's not raw allocated nor
         # pinned.
-    get_nonmovingbuffer._always_inline_ = 'try' # get rid of the returned tuple
-    get_nonmovingbuffer._annenforceargs_ = [strtype]
+    get_nonmovingbuffer_ll._always_inline_ = 'try' # get rid of the returned tuple
+    get_nonmovingbuffer_ll._annenforceargs_ = [strtype]
+
 
     @jit.dont_look_inside
-    def get_nonmovingbuffer_final_null(data):
-        tup = get_nonmovingbuffer(data)
-        buf, flag = tup
+    def get_nonmovingbuffer_ll_final_null(data):
+        tup = get_nonmovingbuffer_ll(data)
+        buf = tup[0]
         buf[len(data)] = lastchar
         return tup
-    get_nonmovingbuffer_final_null._always_inline_ = 'try'
-    get_nonmovingbuffer_final_null._annenforceargs_ = [strtype]
+    get_nonmovingbuffer_ll_final_null._always_inline_ = 'try'
+    get_nonmovingbuffer_ll_final_null._annenforceargs_ = [strtype]
 
-    # (str, char*, char) -> None
+    # args-from-tuple-returned-by-get_nonmoving_buffer() -> None
     # Can't inline this because of the raw address manipulation.
     @jit.dont_look_inside
-    def free_nonmovingbuffer(data, buf, flag):
+    def free_nonmovingbuffer_ll(buf, llobj, flag):
         """
-        Keep 'data' alive and unpin it if it was pinned (flag==\5).
+        Keep 'llobj' alive and unpin it if it was pinned (flag==\5).
         Otherwise free the non-moving copy (flag==\6).
         """
         if flag == '\x05':
-            rgc.unpin(data)
+            rgc.unpin(llobj)
         if flag == '\x06':
             lltype.free(buf, flavor='raw')
         # if flag == '\x04': data was already nonmovable,
         # we have nothing to clean up
-        keepalive_until_here(data)
-    free_nonmovingbuffer._annenforceargs_ = [strtype, None, None]
+        keepalive_until_here(llobj)
 
     # int -> (char*, str, int)
     # Can't inline this because of the raw address manipulation.
@@ -904,12 +949,17 @@ def make_string_mappings(strtype):
         """
         new_buf = mallocfn(count)
         pinned = 0
-        if rgc.can_move(new_buf):
+        fallback = False
+        if rgc.must_split_gc_address_space():
+            fallback = True
+        elif rgc.can_move(new_buf):
             if rgc.pin(new_buf):
                 pinned = 1
             else:
-                raw_buf = lltype.malloc(TYPEP.TO, count, flavor='raw')
-                return raw_buf, new_buf, 2
+                fallback = True
+        if fallback:
+            raw_buf = lltype.malloc(TYPEP.TO, count, flavor='raw')
+            return raw_buf, new_buf, 2
         #
         # following code is executed if:
         # - rgc.can_move(data) and rgc.pin(data) both returned true
@@ -972,26 +1022,122 @@ def make_string_mappings(strtype):
     charpsize2str._annenforceargs_ = [None, int]
 
     return (str2charp, free_charp, charp2str,
-            get_nonmovingbuffer, free_nonmovingbuffer,
-            get_nonmovingbuffer_final_null,
+            get_nonmovingbuffer_ll, free_nonmovingbuffer_ll,
+            get_nonmovingbuffer_ll_final_null,
             alloc_buffer, str_from_buffer, keep_buffer_alive_until_here,
             charp2strn, charpsize2str, str2chararray, str2rawmem,
             )
 
 (str2charp, free_charp, charp2str,
- get_nonmovingbuffer, free_nonmovingbuffer, get_nonmovingbuffer_final_null,
+ get_nonmovingbuffer_ll, free_nonmovingbuffer_ll,
+ get_nonmovingbuffer_ll_final_null,
  alloc_buffer, str_from_buffer, keep_buffer_alive_until_here,
  charp2strn, charpsize2str, str2chararray, str2rawmem,
  ) = make_string_mappings(str)
 
 (unicode2wcharp, free_wcharp, wcharp2unicode,
- get_nonmoving_unicodebuffer, free_nonmoving_unicodebuffer, __not_usable,
+ get_nonmoving_unicodebuffer_ll, free_nonmoving_unicodebuffer_ll, __not_usable,
  alloc_unicodebuffer, unicode_from_buffer, keep_unicodebuffer_alive_until_here,
  wcharp2unicoden, wcharpsize2unicode, unicode2wchararray, unicode2rawmem,
  ) = make_string_mappings(unicode)
 
+
+def constcharp2str(cp):
+    """
+    Like charp2str, but takes a CONST_CCHARP instead
+    """
+    cp = cast(CCHARP, cp)
+    return charp2str(cp)
+constcharp2str._annenforceargs_ = [lltype.SomePtr(CONST_CCHARP)]
+
+
+def constcharpsize2str(cp, size):
+    """
+    Like charpsize2str, but takes a CONST_CCHARP instead
+    """
+    cp = cast(CCHARP, cp)
+    return charpsize2str(cp, size)
+constcharpsize2str._annenforceargs_ = [lltype.SomePtr(CONST_CCHARP), int]
+
+def str2constcharp(s):
+    """
+    Like str2charp, but returns a CONST_CCHARP instead
+    """
+    cp = str2charp(s)
+    return cast(CONST_CCHARP, cp)
+str2constcharp._annenforceargs_ = [str]
+
+@not_rpython
+def _deprecated_get_nonmovingbuffer(*args):
+    raise Exception(
+"""The function rffi.get_nonmovingbuffer() has been removed because
+it was unsafe.  Use rffi.get_nonmovingbuffer_ll() instead.  It returns
+a 3-tuple instead of a 2-tuple, and all three arguments must be passed
+to rffi.free_nonmovingbuffer_ll() (instead of the original string and the
+two tuple items).  Or else, use a high-level API like
+'with rffi.scoped_nonmovingbuffer()'.""")
+
+get_nonmovingbuffer = _deprecated_get_nonmovingbuffer
+get_nonmovingbuffer_final_null = _deprecated_get_nonmovingbuffer
+free_nonmovingbuffer = _deprecated_get_nonmovingbuffer
+
+
+def wcharpsize2utf8(w, size):
+    """ Helper to convert WCHARP pointer to utf8 in one go.
+    Equivalent to wcharpsize2unicode().encode("utf8")
+    Raises rutf8.OutOfRange if characters are outside range(0x110000)!
+    """
+    from rpython.rlib import rutf8
+
+    s = StringBuilder(size)
+    for i in range(size):
+        rutf8.unichr_as_utf8_append(s, ord(w[i]), True)
+    return s.build()
+
+def wcharp2utf8(w):
+    """
+    Raises rutf8.OutOfRange if characters are outside range(0x110000)!
+    """
+    from rpython.rlib import rutf8
+
+    s = rutf8.Utf8StringBuilder()
+    i = 0
+    while ord(w[i]):
+        s.append_code(ord(w[i]))
+        i += 1
+    return s.build(), i
+
+def wcharp2utf8n(w, maxlen):
+    """
+    Raises rutf8.OutOfRange if characters are outside range(0x110000)!
+    """
+    from rpython.rlib import rutf8
+
+    s = rutf8.Utf8StringBuilder(maxlen)
+    i = 0
+    while i < maxlen and ord(w[i]):
+        s.append_code(ord(w[i]))
+        i += 1
+    return s.build(), i
+
+def utf82wcharp(utf8, utf8len, track_allocation=True):
+    from rpython.rlib import rutf8
+
+    if track_allocation:
+        w = lltype.malloc(CWCHARP.TO, utf8len + 1, flavor='raw', track_allocation=True)
+    else:
+        w = lltype.malloc(CWCHARP.TO, utf8len + 1, flavor='raw', track_allocation=False)
+    index = 0
+    for ch in rutf8.Utf8StringIterator(utf8):
+        w[index] = unichr(ch)
+        index += 1
+    w[index] = unichr(0)
+    return w
+utf82wcharp._annenforceargs_ = [str, int, bool]
+
 # char**
 CCHARPP = lltype.Ptr(lltype.Array(CCHARP, hints={'nolength': True}))
+CWCHARPP = lltype.Ptr(lltype.Array(CWCHARP, hints={'nolength': True}))
 
 def liststr2charpp(l):
     """ list[str] -> char**, NULL terminated
@@ -1036,7 +1182,7 @@ def size_and_sign(tp):
     except AttributeError:
         if not isinstance(tp, lltype.Primitive):
             unsigned = False
-        elif tp in (lltype.Signed, FLOAT, DOUBLE, llmemory.Address):
+        elif tp in (lltype.Signed, FLOAT, DOUBLE, LONGDOUBLE, llmemory.Address):
             unsigned = False
         elif tp in (lltype.Char, lltype.UniChar, lltype.Bool):
             unsigned = True
@@ -1058,8 +1204,9 @@ def sizeof(tp):
         if size is None:
             size = llmemory.sizeof(tp)    # a symbolic result in this case
         return size
-    if isinstance(tp, lltype.Ptr) or tp is llmemory.Address:
-        return globals()['r_void*'].BITS/8
+    if (tp is lltype.Signed or isinstance(tp, lltype.Ptr)
+                            or tp is llmemory.Address):
+        return LONG_BIT/8
     if tp is lltype.Char or tp is lltype.Bool:
         return 1
     if tp is lltype.UniChar:
@@ -1072,8 +1219,6 @@ def sizeof(tp):
         # :-/
         return sizeof_c_type("long double")
     assert isinstance(tp, lltype.Number)
-    if tp is lltype.Signed:
-        return LONG_BIT/8
     return tp._type.BITS/8
 sizeof._annspecialcase_ = 'specialize:memo'
 
@@ -1216,15 +1361,29 @@ class scoped_unicode2wcharp:
         if self.buf:
             free_wcharp(self.buf)
 
-
-class scoped_nonmovingbuffer:
-    def __init__(self, data):
-        self.data = data
+class scoped_utf82wcharp:
+    def __init__(self, value, unicode_len):
+        if value is not None:
+            self.buf = utf82wcharp(value, unicode_len)
+        else:
+            self.buf = lltype.nullptr(CWCHARP.TO)
     def __enter__(self):
-        self.buf, self.flag = get_nonmovingbuffer(self.data)
         return self.buf
     def __exit__(self, *args):
-        free_nonmovingbuffer(self.data, self.buf, self.flag)
+        if self.buf:
+            free_wcharp(self.buf)
+
+
+class scoped_nonmovingbuffer:
+
+    def __init__(self, data):
+        self.buf, self.llobj, self.flag = get_nonmovingbuffer_ll(data)
+    __init__._annenforceargs_ = [None, annmodel.SomeString(can_be_None=False)]
+
+    def __enter__(self):
+        return self.buf
+    def __exit__(self, *args):
+        free_nonmovingbuffer_ll(self.buf, self.llobj, self.flag)
     __init__._always_inline_ = 'try'
     __enter__._always_inline_ = 'try'
     __exit__._always_inline_ = 'try'
@@ -1236,25 +1395,24 @@ class scoped_view_charp:
     content of the 'char[]' array will not be modified.
     """
     def __init__(self, data):
-        self.data = data
+        self.buf, self.llobj, self.flag = get_nonmovingbuffer_ll_final_null(
+            data)
     __init__._annenforceargs_ = [None, annmodel.SomeString(can_be_None=False)]
     def __enter__(self):
-        self.buf, self.flag = get_nonmovingbuffer_final_null(self.data)
         return self.buf
     def __exit__(self, *args):
-        free_nonmovingbuffer(self.data, self.buf, self.flag)
+        free_nonmovingbuffer_ll(self.buf, self.llobj, self.flag)
     __init__._always_inline_ = 'try'
     __enter__._always_inline_ = 'try'
     __exit__._always_inline_ = 'try'
 
 class scoped_nonmoving_unicodebuffer:
     def __init__(self, data):
-        self.data = data
+        self.buf, self.llobj, self.flag = get_nonmoving_unicodebuffer_ll(data)
     def __enter__(self):
-        self.buf, self.flag = get_nonmoving_unicodebuffer(self.data)
         return self.buf
     def __exit__(self, *args):
-        free_nonmoving_unicodebuffer(self.data, self.buf, self.flag)
+        free_nonmoving_unicodebuffer_ll(self.buf, self.llobj, self.flag)
     __init__._always_inline_ = 'try'
     __enter__._always_inline_ = 'try'
     __exit__._always_inline_ = 'try'
@@ -1288,10 +1446,91 @@ class scoped_alloc_unicodebuffer:
 c_memcpy = llexternal("memcpy",
             [VOIDP, VOIDP, SIZE_T],
             lltype.Void,
-            releasegil=False
+            releasegil=False,
+            calling_conv='c',
         )
 c_memset = llexternal("memset",
             [VOIDP, lltype.Signed, SIZE_T],
             lltype.Void,
-            releasegil=False
+            releasegil=False,
+            calling_conv='c',
         )
+
+
+# NOTE: This is not a weak key dictionary, thus keeping a lot of stuff alive.
+TEST_RAW_ADDR_KEEP_ALIVE = {}
+
+@jit.dont_look_inside
+def get_raw_address_of_string(string):
+    """Returns a 'char *' that is valid as long as the rpython string object is alive.
+    Two calls to to this function, given the same string parameter,
+    are guaranteed to return the same pointer.
+    """
+    assert isinstance(string, str)
+    from rpython.rtyper.annlowlevel import llstr
+    from rpython.rtyper.lltypesystem.rstr import STR
+    from rpython.rtyper.lltypesystem import llmemory
+    from rpython.rlib import rgc
+
+    if we_are_translated():
+        if rgc.must_split_gc_address_space():
+            return _get_raw_address_buf_from_string(string)
+        if rgc.can_move(string):
+            string = rgc.move_out_of_nursery(string)
+            if rgc.can_move(string):
+                return _get_raw_address_buf_from_string(string)
+
+        # string cannot move now! return the address
+        lldata = llstr(string)
+        data_start = (llmemory.cast_ptr_to_adr(lldata) +
+                      offsetof(STR, 'chars') +
+                      llmemory.itemoffsetof(STR.chars, 0))
+        data_start = cast(CCHARP, data_start)
+        data_start[len(string)] = '\x00'   # write the final extra null
+        return data_start
+    else:
+        global TEST_RAW_ADDR_KEEP_ALIVE
+        if string in TEST_RAW_ADDR_KEEP_ALIVE:
+            return TEST_RAW_ADDR_KEEP_ALIVE[string]
+        result = str2charp(string, track_allocation=False)
+        TEST_RAW_ADDR_KEEP_ALIVE[string] = result
+        return result
+
+class _StrFinalizerQueue(rgc.FinalizerQueue):
+    Class = None              # to use GCREFs directly
+    print_debugging = False   # set to True from test_rffi
+    def finalizer_trigger(self):
+        from rpython.rtyper.annlowlevel import hlstr
+        from rpython.rtyper.lltypesystem import rstr
+        from rpython.rlib import objectmodel
+        while True:
+            gcptr = self.next_dead()
+            if not gcptr:
+                break
+            ll_string = lltype.cast_opaque_ptr(lltype.Ptr(rstr.STR), gcptr)
+            string = hlstr(ll_string)
+            key = objectmodel.compute_unique_id(string)
+            ptr = self.raw_copies.get(key, lltype.nullptr(CCHARP.TO))
+            if ptr:
+                if self.print_debugging:
+                    from rpython.rlib.debug import debug_print
+                    debug_print("freeing str [", ptr, "]")
+                free_charp(ptr, track_allocation=False)
+_fq_addr_from_string = _StrFinalizerQueue()
+_fq_addr_from_string.raw_copies = {}    # {GCREF: CCHARP}
+
+def _get_raw_address_buf_from_string(string):
+    # Slowish but ok because it's not supposed to be used from a
+    # regular PyPy.  It's only used with non-standard GCs like RevDB
+    from rpython.rtyper.annlowlevel import llstr
+    from rpython.rlib import objectmodel
+    key = objectmodel.compute_unique_id(string)
+    try:
+        ptr = _fq_addr_from_string.raw_copies[key]
+    except KeyError:
+        ptr = str2charp(string, track_allocation=False)
+        _fq_addr_from_string.raw_copies[key] = ptr
+        ll_string = llstr(string)
+        gcptr = lltype.cast_opaque_ptr(llmemory.GCREF, ll_string)
+        _fq_addr_from_string.register_finalizer(gcptr)
+    return ptr
