@@ -26,7 +26,10 @@ def dump_on_stdout(database):
 
 def test_primitive():
     db = LowLevelDatabase()
-    assert db.get(5) == '5L'
+    if is_emulated_long:
+        assert db.get(5) == '5LL'
+    else:
+        assert db.get(5) == '5L'
     assert db.get(True) == '1'
 
 def test_struct():
